#!/usr/bin/env python
#-*- coding: utf-8 -*-
u"""
Launches the site's application server.
"""
from woost.models.extension import load_extensions
import --SETUP-PACKAGE--.models
import --SETUP-PACKAGE--.views
from --SETUP-PACKAGE--.controllers import --SETUP-WEBSITE--CMSController

def main():
    load_extensions()
    cms = --SETUP-WEBSITE--CMSController()
    cms.run()

if __name__ == "__main__":
    main()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Item


class TranslationWorkflowComment(Item):

    type_group = "translation_workflow"
    instantiable = False
    visible_from_root = False

    members_order = [
        "request",
        "text"
    ]

    request = schema.Reference(
        type = "woost.extensions.translationworkflow.request."
               "TranslationWorkflowRequest",
        bidirectional = True,
        required = True,
        editable = schema.NOT_EDITABLE
    )

    text = schema.HTML(
        required = True,
        descriptive = True,
        spellcheck = True
    )




#-*- coding: utf-8 -*-
u"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
import os
from shutil import move
import cherrypy
from cocktail.events import event_handler
from cocktail import schema
from cocktail.schema.exceptions import ValidationError
from cocktail.controllers import (
    context,
    session,
    request_property,
    FileUpload
)
from cocktail.controllers.fileupload import FileUpload
from woost import app
from woost.controllers.asyncupload import async_uploader
from woost.controllers.backoffice.editstack import EditNode


class FileEditNode(EditNode):

    @request_property
    def form_adapter(self):
        adapter = EditNode.form_adapter(self)
        adapter.import_rules.add_rule(ImportUploadInfo())
        return adapter

    def get_member_edit_mode(self, member):

        mode = EditNode.get_member_edit_mode(self, member)

        if mode != schema.NOT_EDITABLE and member.name in (
            "mime_type",
            "file_name",
            "file_size"
        ):
            if self.item.is_inserted:
                mode = schema.READ_ONLY
            else:
                mode = schema.NOT_EDITABLE

        return mode

    @request_property
    def form_schema(self):

        form_schema = EditNode.form_schema(self)

        form_schema.add_member(
            FileUpload("upload",
                async = True,
                async_uploader = async_uploader,
                async_upload_url = "/async_upload",
                hash_algorithm = "md5",
                get_file_destination = lambda upload: self.temp_file_path,
                member_group = "content",
                required =
                    not (self.item.is_inserted and self.item.file_name)
            )
        )

        return form_schema

    def iter_changes(self, source = None):

        for member, language in EditNode.iter_changes(self, source):

            # Ignore differences on the upload field if no file has been
            # uploaded
            if member.name == "upload":
                upload = schema.get(self.form_data, "upload", None)
                if (
                    upload is None
                    or upload.get("file_hash") == self.item.file_hash
                ):
                    continue

            yield (member, language)

    @property
    def temp_file_path(self):
        return app.path("upload", "temp", "%s-%s-%s" % (
            self.item_id,
            session.id,
            self.stack.to_param()
        ))

    @event_handler
    def handle_saving(cls, event):

        # Move the uploaded file to its permanent location
        stack_node = event.source
        src = stack_node.temp_file_path

        if os.path.exists(src):

            dest = stack_node.item.file_path

            if os.path.exists(dest):
                os.remove(dest)

            move(src, dest)

        stack_node.form_data["upload"] = None


class ImportUploadInfo(schema.Rule):

    def adapt_object(self, context):

        context.consume("upload")
        upload = context.get("upload", None)

        if upload:
            context.set("file_name", schema.get(upload, "file_name"))
            context.set("mime_type", schema.get(upload, "mime_type"))
            context.set("file_size", schema.get(upload, "file_size"))
            context.set("file_hash", schema.get(upload, "file_hash"))


class FileRequiredError(ValidationError):

    @property
    def invalid_members(self):
        return self.member["upload"]




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.events import event_handler
from cocktail.translations import translations
from woost.controllers.backoffice.editstack import EditNode
from woost.controllers.notifications import Notification


class CampaignMonitorListEditNode(EditNode):

    @event_handler
    def handle_committed(cls, event):

        item = event.source.item
        try:
            item.update()
        except Exception, e:
            Notification(
                translations(
                    "woost.extensions.campaignmonitorlisteditnode.CampaignMonitorListEditNode failed synchronization",
                    item = item,
                    exception = e
                ),
                "error"
            ).emit()
        else:
            Notification(
                translations(
                    "woost.extensions.campaignmonitorlisteditnode.CampaignMonitorListEditNode synchronized changes",
                    item = item
                ),
                "success"
            ).emit()



#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2013
"""
import urllib2
import cherrypy
from cocktail.events import event_handler
from cocktail import schema
from cocktail.translations import translations
from cocktail.persistence import transactional
from cocktail.controllers import request_property
from woost import app
from woost.models import (
    changeset_context,
    File,
    InstallationSyncPermission
)
from woost.models.synchronization import Synchronization
from woost.controllers.notifications import Notification
from woost.controllers.backoffice.editcontroller import EditController
from woost.controllers.backoffice.useractions import get_user_action


class InstallationSyncController(EditController):

    view_class = "woost.views.BackOfficeInstallationSyncView"

    @event_handler
    def handle_traversed(cls, e):
        app.user.require_permission(InstallationSyncPermission)

    @request_property
    def synchronization(self):
        site_installation = self.context["cms_item"]
        return Synchronization(
            site_installation.url,
            site_installation.synchronization_user,
            site_installation.synchronization_password
        )

    @request_property
    def comparision(self):
        try:
            return self.synchronization.compare_content(
                establish_relations = (self.sync_action == "sync")
            )
        except urllib2.HTTPError, e:
            self.sync_request_error = e
            return {"incomming": (), "modified": {}}

    @request_property
    def sync_request_error(self):
        return None

    @request_property
    def sync_action(self):
        return cherrypy.request.params.get("sync_action")

    @request_property
    def sync_selection(self):
        return cherrypy.request.params.get("sync_selection") or ()

    @request_property
    def submitted(self):
        return EditController.submitted(self) or self.sync_action

    def submit(self):
        if self.sync_action == "sync" and self.sync_request_error is None:
            self._synchronize()
        else:
            EditController.submit(self)

    @transactional()
    def _synchronize(self):
        cmp = self.comparision
        get_param = cherrypy.request.params.get
        file_downloads = set()
        selection = self.sync_selection

        with changeset_context(app.user):

            for obj in cmp["incomming"]:
                if obj.global_id in selection:
                    obj.insert()

                    if isinstance(obj, File):
                        file_downloads.add(obj)

            for global_id, change in cmp["modified"].iteritems():
                if global_id in selection:
                    local = change["local"]
                    remote = change["remote"]

                    for member, lang in change["diff"]:
                        value = schema.get(remote, member, language = lang)
                        local.set(member, value, lang)

                        if member in (File.file_hash, File.file_size):
                            file_downloads.add(local)

        # Download files from the remote copy
        for file in file_downloads:
            self.synchronization.download_file(file)

        # Find importable content again
        self.comparision = self.synchronization.compare_content()

        Notification(
            translations("woost.controllers.InstallationSyncController.success"),
            "success"
        ).emit()

    @request_property
    def output(self):
        output = EditController.output(self)
        output.update(self.comparision)
        output.update(
            selected_action = get_user_action("installation_sync"),
            sync_selection = self.sync_selection,
            sync_request_error = self.sync_request_error
        )
        return output




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Configuration, LocaleMember
from .customdefinition import GoogleAnalyticsCustomDefinition

Configuration.add_member(
    schema.String("google_analytics_account",
        text_search = False,
        member_group = "services.google_analytics",
        synchronizable = False,
        listed_by_default = False
    )
)

Configuration.add_member(
    schema.String("google_analytics_domain",
        text_search = False,
        member_group = "services.google_analytics",
        synchronizable = False,
        listed_by_default = False
    )
)

Configuration.add_member(
    LocaleMember("google_analytics_language",
        member_group = "services.google_analytics",
        synchronizable = False,
        listed_by_default = False
    )
)

Configuration.add_member(
    schema.Collection("google_analytics_custom_definitions",
        items = schema.Reference(type = GoogleAnalyticsCustomDefinition),
        related_end = schema.Reference(),
        member_group = "services.google_analytics",
        synchronizable = False,
        listed_by_default = False
    )
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail.html import templates
from woost.models import Extension


translations.define("AnnotationsExtension",
    ca = u"Annotacions",
    es = u"Anotaciones",
    en = u"Annotations"
)

translations.define("AnnotationsExtension-plural",
    ca = u"Annotacions",
    es = u"Anotaciones",
    en = u"Annotations"
)


class AnnotationsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet afegir annotacions als camps durant l'edició.""",
            "ca"
        )
        self.set("description",
            u"""Permite añadir anotaciones a los campos durante la edición.""",
            "es"
        )
        self.set("description",
            u"""Annotate editable fields.""",
            "en"
        )

    def _load(self):

        from woost.extensions.annotations import (
            strings,
            annotations
        )

        templates.get_class("woost.extensions.annotations.BackOfficeFieldsViewOverlay")




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.modeling import cached_getter
from cocktail import schema
from cocktail.html import templates
from cocktail.html.uigeneration import display_factory
from cocktail.html.readonlydisplay import read_only_display
from woost.models import Item
from .path import TranslationWorkflowPath
from .transitionsetup import TranslationWorkflowTransitionSetup


class TranslationWorkflowAssignTransitionSetup(
    TranslationWorkflowTransitionSetup
):
    @cached_getter
    def multiple_choices(self):
        return any(
            len(translators) != 1
            for translators in self.eligible_translators.itervalues()
        )

    @cached_getter
    def transition_schema(self):

        transition_schema = schema.Schema(
            "woost.extensions.translationworkflow."
            "TranslationWorkflowAssignTransitionSetup"
        )

        def translator_edit_control(
            ui_generator,
            obj,
            member,
            value,
            **context
        ):
            selector = templates.new("cocktail.html.DropdownSelector")
            translation_path = context.get("item_key")
            if translation_path is not None:
                selector.items = \
                    self.eligible_translators.get(translation_path)
            return selector

        assignments = schema.Mapping("assignments",
            keys = TranslationWorkflowPath("translation_path",
                required = True,
                edit_control = read_only_display
            ),
            values = schema.Reference("translator",
                type = Item,
                required = True,
                # Limit the translator selection to translators who are able to
                # deal with each particular path
                enumeration = lambda ctx:
                    self.eligible_translators.get(ctx.collection_index),
                edit_control = translator_edit_control
            ),
            edit_control = display_factory(
                "cocktail.html.MappingEditor",
                fixed_entries = True
            )
        )

        # Make each path default to its first eligible translator
        assignments.default = {}

        for translation_path in self.translation_paths:
            path_translators = self.eligible_translators.get(translation_path)
            if path_translators:
                default_translator = path_translators[0]
            else:
                default_translator = None
            assignments.default[translation_path] = default_translator

        transition_schema.add_member(assignments)
        return transition_schema




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from json import dumps
from cocktail.html import Element
from .utils import get_ga_custom_values

Element.ga_event_category = None
Element.ga_event_action = None
Element.ga_event_label = None
Element.ga_event_value = None
Element.ga_event_source = None
Element.ga_event_env = None
Element.ga_event_overrides = None

Element._ga_event_data = None

def _get_element_ga_event_data(self):
    if self._ga_event_data is None and (
        self.ga_event_source
        or self.ga_event_overrides
    ):
        return get_ga_custom_values(
            self.ga_event_source,
            overrides = self.ga_event_overrides,
            env = self.ga_event_env
        )
    return self._ga_event_data

def _set_element_ga_event_data(self, data):
    self._ga_event_data = data

Element.ga_event_data = property(
    _get_element_ga_event_data,
    _set_element_ga_event_data
)

_base_ready_element = Element._ready

def _ready_element(self):

    _base_ready_element(self)

    self["data-ga-event-category"] = self.ga_event_category
    self["data-ga-event-action"] = self.ga_event_action
    self["data-ga-event-label"] = self.ga_event_label
    self["data-ga-event-value"] = self.ga_event_value

    data = self.ga_event_data
    if data:
        self["data-ga-event-data"] = dumps(data)

Element._ready = _ready_element




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Block, Publishable


class NewsletterListing(Block):

    type_group = "blocks.newsletter"

    members_order = [
        "view_class",
        "listed_items"
    ]

    view_class = schema.String(
        shadows_attribute = True,
        default = "woost.extensions.newsletters.NewsletterListingTextOnlyView",
        enumeration = [
            "woost.extensions.newsletters.NewsletterListingTextOnlyView",
            "woost.extensions.newsletters.NewsletterListingTextAndIconView",
            "woost.extensions.newsletters.NewsletterListingSummaryView"
        ],
        required = True,
        text_search = False,
        member_group = "content"
    )

    listed_items = schema.Collection(
        items = schema.Reference(type = Publishable),
        related_end = schema.Collection(),
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.listed_items = self.listed_items




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.events import when
from cocktail.persistence import MigrationStep

step = MigrationStep(
    "woost.extensions.variables: Rename Variable.expression to .dynamic_value"
)

@when(step.executing)
def rename_variable_expression_member(e):

    from woost.extensions.variables.variable import Variable

    for variable in Variable.select():
        try:
            value = variable._expression
        except AttributeError:
            continue
        else:
            del variable._expression
            variable.dynamic_value = value




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Pepe Osca <pepe.osca@whads.com>
"""
import json
import urllib
import urllib2
from cocktail.styled import styled
from cocktail import schema
from cocktail.controllers import Location
from woost import app
from woost.models import Configuration
from woost.extensions.identity.identityprovider import IdentityProvider


class GoogleIdentityProvider(IdentityProvider):

    provider_name = "Google"
    user_identifier = "google_user_id"

    members_order = [
        "client_id",
        "client_secret",
        "scope",
        "access_type"
    ]

    client_id = schema.String(
        required = True,
        text_search = False
    )

    client_secret = schema.String(
        required = True,
        text_search = False
    )

    scope = schema.Collection(
        min = 1,
        default = schema.DynamicDefault(lambda: ["profile", "email"]),
        items = schema.String()
    )

    access_type = schema.String(
        required = True,
        default = "online",
        enumeration = ["online", "offline"],
        edit_control = "cocktail.html.RadioSelector"
    )

    def get_auth_url(self, target_url = None):
        return (
            "/google_oauth/%d/step1?%s" % (
                self.id,
                urllib.urlencode({
                    "target_url": target_url
                                  or unicode(
                        Location.get_current(relative=False)).encode("utf-8")
                })
            )
        )

    def get_refresh_token_url(self, redirect_uri = None):
        """
        Produces the URL to initiate a browser driven request to get a refresh
        code.
        """
        OAUTH_URL = "https://accounts.google.com/o/oauth2/auth"

        if not redirect_uri:
            website = (app.website or Configuration.instance.websites[0])
            redirect_uri = "http://%s/google_oauth/%d/refresh_token" % (
                website.hosts[0],
                self.id
            )

        params = {
            "client_id": self.client_id,
            "redirect_uri": redirect_uri,
            "scope": " ".join(self.scope),
            "response_type": "code",
            "access_type": self.access_type,
            "approval_prompt": "force"
        }

        return "{}?{}".format(OAUTH_URL, urllib.urlencode(params))

    def get_refresh_token_from_browser(self, redirect_uri = None):
        """
        Opens the browser to get user authorization and shows the
        authorization code.
        """
        import webbrowser
        url = self.get_refresh_token_url(redirect_uri)
        webbrowser.open(url)

    def get_refresh_token(self, code, redirect_uri = None):
        """
        Gets an authorization token with access_token and refresh_token from
        an authorization code.

        :param code: A valid authorization code with access_type offline
        :return: A dict representing the authorization token. The
        'access_token' key is a valid access token and the 'refresh_token'
        key can be used to generate new valid access tokens
        """
        TOKEN_URL = "https://accounts.google.com/o/oauth2/token"

        if not redirect_uri:
            website = (app.website or Configuration.instance.websites[0])
            redirect_uri = "http://%s/google_oauth/%d/refresh_token" % (
                website.hosts[0],
                self.id
            )

        params = {
            "code": code,
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "redirect_uri": redirect_uri,
            "grant_type": "authorization_code"
        }

        request = urllib2.Request(url=TOKEN_URL, data=urllib.urlencode(params))
        json_file = urllib2.urlopen(request).read()
        auth_token = json.loads(json_file)

        if self.debug_mode:
            print styled("Google refresh token:", "magenta"), auth_token

        return auth_token

    def get_access_token_from_refresh_token(self, refresh_token):
        """
        Gets a new access token from a refresh token.

        :return: A dict respresenting an access token. Its 'access_token' key
        is a valid access token
        """
        TOKEN_URL = "https://accounts.google.com/o/oauth2/token"

        params = {
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "refresh_token": refresh_token,
            "grant_type": "refresh_token"
        }

        request = urllib2.Request(url=TOKEN_URL, data=urllib.urlencode(params))

        try:
            json_file = urllib2.urlopen(request).read()
        except urllib2.HTTPError, e:
            if self.debug_mode:
                print styled(
                    "Error while obtaining a Google access token from a "
                    "refresh token:",
                    "magenta"
                )
                print e.read()
            raise

        access_token = json.loads(json_file)

        if self.debug_mode:
            print styled("Google access token from refresh token:", "magenta"),
            print access_token

        return access_token




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.tests.models.basetestcase import BaseTestCase


class WebsiteHostMatchingTestCase(BaseTestCase):

    def test_can_match_more_than_one_host(self):

        from woost.models import Website

        w1 = Website()
        w1.hosts.append("foo.com")
        w1.hosts.append("foo.net")
        self.config.websites.append(w1)

        w2 = Website()
        w2.hosts.append("bar.com")
        w2.hosts.append("spam.bar.com")
        self.config.websites.append(w2)

        assert self.config.get_website_by_host("foo.com") is w1
        assert self.config.get_website_by_host("foo.net") is w1
        assert self.config.get_website_by_host("bar.com") is w2
        assert self.config.get_website_by_host("spam.bar.com") is w2

    def test_solves_collissions_by_relative_order_of_websites_in_configuration(self):

        from woost.models import Website

        w1 = Website()
        w1.hosts.append("bar.com")
        w1.hosts.append("foo.com")
        self.config.websites.append(w1)

        w2 = Website()
        w2.hosts.append("foo.com")
        self.config.websites.append(w2)

        assert self.config.get_website_by_host("foo.com") is w1


class WebsiteLanguageTestCase(BaseTestCase):

    def test_websites_can_override_language_visibility(self):

        from woost import app
        from woost.models import Configuration, Website

        w1 = Website()
        w2 = Website()

        self.config.languages = ["ca", "es", "en"]

        for website in (None, w1, w2):
            app.website = website
            for lang in ["ca", "es", "en"]:
                assert self.config.language_is_enabled(lang)

        self.config.published_languages = ["en"]

        for website in (None, w1, w2):
            app.website = website
            assert self.config.language_is_enabled("en")
            assert not self.config.language_is_enabled("ca")
            assert not self.config.language_is_enabled("es")

        w1.published_languages = ["ca"]

        app.website = w1
        assert self.config.language_is_enabled("ca")
        assert not self.config.language_is_enabled("es")
        assert not self.config.language_is_enabled("en")

        app.website = w2
        assert self.config.language_is_enabled("en")
        assert not self.config.language_is_enabled("ca")
        assert not self.config.language_is_enabled("es")


class WebsiteSpecificContentTestCase(BaseTestCase):

    def test_descendants_inherit_availability(self):

        from woost.models import Website, Document

        w1 = Website()
        w2 = Website()

        d1 = Document()
        d1.websites = {w1}

        d2 = Document()
        d2.websites = {w2}
        d2.parent = d1
        assert set(d2.websites) == {w1}

        d3 = Document()
        d3.parent = d2
        assert set(d3.websites) == {w1}

        d1.websites.add(w2)
        assert set(d2.websites) == {w1, w2}
        assert set(d3.websites) == {w1, w2}

        d1.websites = set()
        assert not d2.websites
        assert not d3.websites

    def test_designating_a_website_home_implies_exclusivity(self):

        from woost.models import Website, Document

        d1 = Document()
        w1 = Website()
        w1.home = d1
        assert set(d1.websites) == {w1}

    def test_websites_own_document_trees_exclusively(self):

        from woost.models import Website, Document

        d1 = Document()

        w1 = Website()
        w1.home = d1

        w2 = Website()
        w2.home = d1

        assert set(d1.websites) == {w2}


class WebsiteSpecificContentIndexingTestCase(BaseTestCase):

    def test_content_is_not_indexed_until_inserted(self):

        from woost.models import Website, Publishable

        index = Publishable.per_website_publication_index

        w1 = Website()
        w1.insert()

        p1 = Publishable()
        assert not list(index.items())

        p1.websites = {w1}
        assert not list(index.items())

    def test_content_is_not_indexed_until_website_is_inserted(self):

        from woost.models import Website, Publishable

        index = Publishable.per_website_publication_index

        w1 = Website()

        p1 = Publishable()
        p1.insert()
        p1.websites = {w1}

        assert (w1.require_id(), p1.id) not in list(index.items())

    def test_content_is_published_for_all_websites_by_default(self):

        from woost.models import Website, Publishable

        index = Publishable.per_website_publication_index

        w1 = Website()
        w1.insert()

        p1 = Publishable()
        p1.insert()

        assert list(index.items()) == [(None, p1.id)]

        p2 = Publishable()
        p2.insert()

        assert set(index.items()) == set([(None, p1.id), (None, p2.id)])

    def test_content_can_change_its_designated_websites(self):

        from woost.models import Website, Publishable

        index = Publishable.per_website_publication_index

        p1 = Publishable()
        p1.insert()

        w1 = Website()
        w1.insert()

        w2 = Website()
        w2.insert()

        assert list(index.items()) == [(None, p1.id)]

        p1.websites = {w1}
        assert list(index.items()) == [(w1.id, p1.id)]

        p1.websites = {w2}
        assert list(index.items()) == [(w2.id, p1.id)]

        p1.websites = {w1, w2}
        assert set(index.items()) == set([(w1.id, p1.id), (w2.id, p1.id)])

        p1.websites = set()
        assert list(index.items()) == [(None, p1.id)]

    def test_index_is_updated_after_deleting_publishable(self):

        from woost.models import Website, Publishable

        index = Publishable.per_website_publication_index

        p1 = Publishable()
        p1.insert()
        p1.delete()

        assert (None, p1.id) not in list(index.items())

        w1 = Website()
        w1.insert()

        p2 = Publishable()
        p2.websites = {w1}
        p2.insert()
        p2.delete()

        assert not list(index.items())

    def test_index_is_updated_after_deleting_website(self):

        from woost.models import Website, Publishable

        index = Publishable.per_website_publication_index

        w1 = Website()
        w1.insert()

        w2 = Website()
        w2.insert()

        p1 = Publishable()
        p1.websites = {w1}
        p1.insert()

        p2 = Publishable()
        p2.websites = {w1, w2}
        p2.insert()

        w1.delete()
        assert set(index.items()) == set([(None, p1.id), (w2.id, p2.id)])

        w2.delete()
        assert set(index.items()) == set([(None, p1.id), (None, p2.id)])




#-*- coding: utf-8 -*-
"""Provides functions for dealing with user notifications.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from warnings import warn
from cocktail.controllers import session
from cocktail.html import templates

def notify_user(message, category = None, transient = True):
    """Creates a new notification for the current user.

    Notifications are stored until a proper page is served to the user. It
    is up to the views to decide how these messages should be displayed.

    @param message: The message that will be shown to the user.
    @type message: unicode

    @param category: An optional free form string identifier that qualifies
        the message. Standard values include 'success' and 'error'.
    @type category: unicode

    @param transient: Indicates if the message should be hidden after a
        short lapse (True), or if it should remain visible until explicitly
        closed by the user (False).
    @type transient: bool
    """
    warn(
        "notify_user() is deprecated; use the Notification class instead",
        DeprecationWarning,
        stacklevel = 2
    )
    Notification(message, category, transient).emit()

def pop_user_notifications(filter = None):
    warn(
        "pop_user_notifications() is deprecated; "
        "use the Notification class instead",
        DeprecationWarning,
        stacklevel = 2
    )
    return Notification.pop(filter)


class Notification(object):

    def __init__(self, message = None, category = None, transient = False):
        self.message = message
        self.category = category
        self.transient = transient

    def __len__(self):
        # Required for backwards compatibility; notifications used to be tuples
        return 3

    def __iter__(self):
        # Required for backwards compatibility; notifications used to be tuples
        warn(
            "Notifications are no longer tuples; used the provided attributes "
            "instead",
            DeprecationWarning,
            stacklevel = 2
        )
        yield self.message
        yield self.category
        yield self.transient

    def __getitem__(self, index):
        # Required for backwards compatibility; notifications used to be tuples
        warn(
            "Notifications are no longer tuples; used the provided attributes "
            "instead",
            DeprecationWarning,
            stacklevel = 2
        )
        if index == 0:
            return self.message
        elif index == 1:
            return self.category
        elif index == 2:
            return self.transient

    def emit(self):
        """Stores the notification, so it can be retrieved by a call to
        L{pop}.
        """
        notifications = session.get("notifications")
        if notifications is None:
            session["notifications"] = [self]
        else:
            notifications.append(self)

    @classmethod
    def pop(cls, filter = None):
        """Retrieves pending notification messages that were stored through the
        L{notify_user} method.

        Retrieved messages are considered to be consumed, and therefore they
        are removed from the list of pending notifications.

        @param filter: If given, only those notifications matching the
            specified criteria will be retrieved. The criteria matches as
            follows:

                * If set to a string, it designates a single notification
                  category to retrieve.
                * If set to a subclass of L{Notification}, only notifications
                  that are instances of that type will be retrieved.
                * If set to a tuple of subclasses of L{Notification}, only
                  notifications that are instances of any of the indicated
                  types will be retrieved.
                * If set to a callable, it will be used as filter function,
                  taking a L{Notification} as its single parameter, and
                  returning True if the notification is to be retrieved, or
                  False if it should be ignored.
                * If set to a sequence (list, tuple, set), it designates a set
                  of categories; notifications belonging to any of the
                  specified categories will be retrieved.

        @return: The list of pending notification messages.
        @rtype: L{Notification} list
        """
        notifications = session.get("notifications")

        if notifications is None:
            return []

        remaining = []

        if filter:
            matching = []

            if isinstance(filter, basestring):
                match = lambda notification: notification.category == filter
            elif (
                isinstance(filter, type) and issubclass(filter, Notification)
                or (
                    isinstance(filter, tuple)
                    and filter
                    and isinstance(filter[0], type)
                    and issubclass(filter[0], Notification)
                )
            ):
                match = lambda notification: isinstance(notification, filter)
            elif callable(filter):
                match = filter
            elif hasattr("__contains__", filter):
                match = lambda notification: notification.category in filter
            else:
                raise TypeError(
                    "The 'filter' parameter for the pop_user_notifications() "
                    "function should be a string, a sequence or callable, got "
                    "%s instead" % type(filter)
                )

            for notification in notifications:
                (matching if match(notification) else remaining).append(notification)

            notifications = matching

        session["notifications"] = remaining
        return notifications

    view_class = "woost.views.Notification"

    def create_view(self):
        view = templates.new(self.view_class)
        view.notification = self
        return view




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models import Block
from .utils import escape_ga_string, get_ga_value

base_init_view = Block.init_view

def init_view(self, view):

    if not view.ga_event_category:
        view.ga_event_category = get_ga_value(self.__class__)

    if not view.ga_event_action:
        view.ga_event_action = "click"

    if not view.ga_event_label:
        view.ga_event_label = get_ga_value(self)

    base_init_view(self, view)

Block.init_view = init_view




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from cocktail.html import templates
from cocktail.html.uigeneration import display_factory
from woost.models import Item

ContentList = templates.get_class("woost.views.ContentList")


class ImageCollectionDisplay(ContentList):

    icon_factory = "backoffice_small_thumbnail.png"

    def __init__(self, *args, **kwargs):
        ContentList.__init__(self, *args, **kwargs)

        def thumbnail(ui_generator, obj, member, value, **context):
            display = templates.new("woost.views.ItemDisplay")
            display.label_visible = False
            display.icon_visible = True
            display.icon_factory = self.icon_factory
            return display

        self.set_member_type_display(Item, thumbnail)

    def create_member_display(self, *args, **kwargs):
        return ContentList.create_member_display(self, *args, **kwargs)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from cocktail import schema
from cocktail.controllers.imageupload import get_image_size
from woost.models.file import File
from woost.models.rendering.renderer import Renderer
from woost.models.rendering.formats import formats_by_mime_type


class ImageFileRenderer(Renderer):
    """A content renderer that handles image files."""

    instantiable = True

    max_size = schema.Tuple(
        items = (
            schema.Integer(
                required = True,
                min = 1
            ),
            schema.Integer(
                required = True,
                min = 1
            )
        ),
        request_value_separator = "x"
    )

    def can_render(self, item, **parameters):
        return (
            isinstance(item, File)
            and item.resource_type == "image"
            and item.mime_type in formats_by_mime_type
            and (self.max_size is None or self.validate_size(item))
        )

    def validate_size(self, file):
        try:
            width, height = get_image_size(file.file_path)
        except IOError:
            return False
        else:
            max_size = self.max_size
            return (width <= max_size[0] and height <= max_size[1]) \
                or (width <= max_size[1] and height <= max_size[0])

    def render(self, item, **parameters):
        return item.file_path




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.events import event_handler
from cocktail.modeling import extend, call_base
from cocktail.translations import translations
from cocktail import schema
from cocktail.html.datadisplay import display_factory
from woost.models import Item


class Field(Item):

    visible_from_root = False
    show_element_in_listing = False
    type_group = "custom_forms"
    instantiable = False
    member_type = None

    groups_order = [
        "field_description",
        "field_structure",
        "field_properties"
    ]

    members_order = [
        "title",
        "visible_title",
        "empty_label",
        "explanation",
        "field_set",
        "collection",
        "is_required_field",
        "field_edit_control",
        "field_initialization"
    ]

    title = schema.String(
        translated = True,
        required = True,
        descriptive = True,
        spellcheck = True,
        member_group = "field_description"
    )

    visible_title = schema.String(
        translated = True,
        member_group = "field_description",
        listed_by_default = False
    )

    empty_label = schema.String(
        translated = True,
        member_group = "field_description",
        listed_by_default = False
    )

    explanation = schema.HTML(
        translated = True,
        spellcheck = True,
        edit_control = "woost.views.RichTextEditor",
        member_group = "field_description"
    )

    field_name = schema.String(
        member_group = "field_structure",
        listed_by_default = False
    )

    field_set = schema.Reference(
        type = "woost.extensions.forms.fields.FieldSet",
        bidirectional = True,
        editable = schema.NOT_EDITABLE,
        member_group = "field_structure"
    )

    collection = schema.Reference(
        type = "woost.extensions.forms.fields.CollectionField",
        bidirectional = True,
        editable = schema.NOT_EDITABLE,
        member_group = "field_structure",
        listed_by_default = False
    )

    is_required_field = schema.Boolean(
        required = True,
        default = True,
        member_group = "field_properties",
        listed_by_default = False
    )

    field_edit_controls = None

    field_edit_control = schema.String(
        enumeration = lambda ctx:
            ctx.get("persistent_object")
            and ctx.get("persistent_object").field_edit_controls,
        edit_control = display_factory(
            "cocktail.html.RadioSelector",
            empty_option_displayed = True
        ),
        member_group = "field_properties",
        listed_by_default = False
    )

    field_initialization = schema.CodeBlock(
        language = "python",
        listed_by_default = False,
        member_group = "field_properties"
    )

    @event_handler
    def handle_inherited(cls, e):
        # Add members that depend on the field's type (f. eg. although all
        # fields can have a default value, we can't add the member to represent
        # it until we know the field's type)
        field_class = e.schema
        member_type = getattr(field_class, "member_type", None)
        if member_type is not None:
            field_class.add_generic_members()

    @classmethod
    def add_generic_members(cls):

        # Declare a member for the field's default value
        if cls.get_member("field_default") is None:
            default_member = cls.create_field_default_member()
            if default_member is not None:
                cls.add_member(default_member)

        # Declare a member for the field's enumeration
        if cls.get_member("field_enumeration") is None:
            enumeration_member = cls.create_field_enumeration_member()
            if enumeration_member is not None:
                cls.add_member(enumeration_member)

    @classmethod
    def create_field_default_member(cls):

        default_member = cls.member_type(
            "field_default",
            member_group = "field_properties",
            listed_by_default = False
        )

        @extend(default_member)
        def __translate__(default_member, language, **kwargs):
            return translations(
                "woost.extensions.forms.field_default",
                language,
                **kwargs
            )

        return default_member

    @classmethod
    def create_field_enumeration_member(cls):

        enumeration_member = schema.Collection(
            "field_enumeration",
            items = cls.member_type(),
            member_group = "field_properties",
            listed_by_default = False
        )

        @extend(enumeration_member)
        def __translate__(enumeration_member, language, **kwargs):
            return translations(
                "woost.extensions.forms.field_enumeration",
                language,
                **kwargs
            )

        return enumeration_member

    def generate_members(self):
        yield self.create_member()

    def create_member(self):

        member = self.member_type(
            self.field_name or "field%d" % self.id
        )
        self._init_member(member)

        if self.field_initialization:
            label = "%s #%s.field_initialization" % (
                self.__class__.__name__,
                self.id
            )
            context = {"field": self, "member": member}
            code = compile(self.field_initialization, label, "exec")
            exec code in context

        return member

    def _init_member(self, member):

        member.required = self.is_required_field

        if self.__class__.get_member("field_default") is not None:
            member.default = self.field_default

        if self.__class__.get_member("field_enumeration") is not None:
            member.enumeration = self.field_enumeration or None

        member.edit_control = self.field_edit_control

        @extend(member)
        def __translate__(member, language, **kwargs):
            suffix = kwargs.get("suffix")
            if suffix == "=none":
                return self.get("empty_label", language)
            elif suffix:
                return call_base(language, **kwargs)
            else:
                return (
                    self.get("visible_title", language)
                    or self.get("title", language)
                )

        @extend(member)
        def get_member_explanation(member, language = None, **kwargs):
            return self.get("explanation", language)


class FieldSet(Field):

    visible_from_root = True
    member_type = schema.Schema
    members_order = [
        "base_field_set",
        "derived_field_sets",
        "fields"
    ]

    base_field_set = schema.Reference(
        type = "woost.extensions.forms.fields.FieldSet",
        bidirectional = True,
        member_group = "field_structure",
        listed_by_default = False
    )

    derived_field_sets = schema.Collection(
        items = "woost.extensions.forms.fields.FieldSet",
        bidirectional = True,
        editable = schema.NOT_EDITABLE,
        member_group = "field_structure",
        listed_by_default = False
    )

    fields = schema.Collection(
        items = "woost.extensions.forms.fields.Field",
        bidirectional = True,
        member_group = "field_structure",
        listed_by_default = False
    )

    @classmethod
    def create_field_default_member(cls):
        pass

    @classmethod
    def create_field_enumeration_member(cls):
        pass

    def generate_members(self):
        for field in self.fields:
            for member in field.generate_members():
                if not member.member_group:
                    member.member_group = str(self.id)
                else:
                    member.member_group = str(self.id) + "." + member.member_group
                yield member

    def _init_member(self, member):

        Field._init_member(self, member)

        if self.base_field_set:
            member.inherit(self.base_field_set.create_member())

        for child_member in self.generate_members():
            member.add_member(child_member, append = True)

        @extend(member)
        def translate_group(member, group):
            try:
                field_set_id = int(group.split(".")[-1])
            except:
                return call_base(group)
            else:
                return translations(FieldSet.require_instance(field_set_id))

    def create_form_model(self):

        # Create the user defined schema
        form_model = self.create_member()

        # Add the "submit date" member
        submit_date = schema.DateTime("submit_date")

        @extend(submit_date)
        def __translate__(member, language, **kwargs):
            return translations(
                "woost.extensions.forms.submit_date",
                language,
                **kwargs
            )

        form_model.add_member(submit_date)

        return form_model


class CollectionField(Field):

    visible_from_root = False

    members_order = [
        "items",
        "min",
        "max"
    ]

    member_type = schema.Collection

    field_edit_controls = [
        "cocktail.html.CollectionEditor",
        "cocktail.html.CheckList"
    ]

    items = schema.Reference(
        type = "woost.extensions.forms.fields.Field",
        bidirectional = True,
        integral = True,
        required = True,
        listed_by_default = False
    )

    min = schema.Integer(
        min = 0,
        listed_by_default = False
    )

    max = schema.Integer(
        min = min,
        listed_by_default = False
    )

    @classmethod
    def create_field_default_member(cls):
        pass

    @classmethod
    def create_field_enumeration_member(cls):
        pass

    def _init_member(self, member):
        Field._init_member(self, member)
        member.items = self.items.create_member()
        member.min = self.min
        member.max = self.max


class BooleanField(Field):

    visible_from_root = False
    member_type = schema.Boolean

    field_edit_controls = [
        "cocktail.html.CheckBox",
        "cocktail.html.RadioSelector"
    ]


class TextField(Field):

    visible_from_root = False
    member_type = schema.String

    field_edit_controls = [
        "cocktail.html.TextBox",
        "cocktail.html.TextArea"
    ]

    members_order = [
        "format",
        "min",
        "max"
    ]

    format = schema.String(
        member_group = "field_properties",
        listed_by_default = False
    )

    min = schema.Integer(
        min = 0,
        member_group = "field_properties",
        listed_by_default = False
    )

    max = schema.Integer(
        min = min,
        member_group = "field_properties",
        listed_by_default = False
    )

    def _init_member(self, member):
        Field._init_member(self, member)
        member.format = self.format
        member.min = self.min
        member.max = self.max


class OptionsFieldOption(Item):

    visible_from_root = False
    show_element_in_listing = False

    members_order = [
        "field",
        "title"
    ]

    field = schema.Reference(
        type = "woost.extensions.forms.fields.OptionsField",
        bidirectional = True,
        required = True,
        listed_by_default = False
    )

    title = schema.String(
        required = True,
        spellcheck = True,
        descriptive = True,
        translated = True
    )


class OptionsField(Field):

    visible_from_root = False
    member_type = schema.Reference

    field_edit_controls = [
        "cocktail.html.DropdownSelector",
        "cocktail.html.RadioSelector"
    ]

    options = schema.Collection(
        items = "woost.extensions.forms.fields.OptionsFieldOption",
        bidirectional = True,
        integral = True,
        min = 1,
        member_group = "field_properties"
    )

    @classmethod
    def create_field_default_member(cls):
        default_member = Field.create_field_default_member.im_func(cls)
        default_member.type = OptionsFieldOption
        default_member.enumeration = cls.options
        return default_member

    @classmethod
    def create_field_enumeration_member(cls):
        pass

    def _init_member(self, member):
        Field._init_member(self, member)
        member.type = OptionsFieldOption
        member.enumeration = lambda ctx: self.options


class EmailAddressField(Field):
    visible_from_root = False
    member_type = schema.EmailAddress


class PhoneNumberField(Field):
    visible_from_root = False
    member_type = schema.PhoneNumber


class HTMLField(TextField):
    visible_from_root = False
    member_type = schema.HTML
    field_edit_controls = None


class NumericField(Field):

    instantiable = False
    visible_from_root = False

    members_order = [
        "min",
        "max"
    ]

    @classmethod
    def add_generic_members(cls):
        Field.add_generic_members.im_func(cls)
        if cls.get_member("min") is None:
            min_member = cls.member_type(
                "min",
                listed_by_default = False
            )
            max_member = cls.member_type("max",
                min = min_member,
                listed_by_default = False
            )
            cls.add_member(min_member)
            cls.add_member(max_member)

    def _init_member(self, member):
        Field._init_member(self, member)
        member.min = self.min
        member.max = self.max


class IntegerField(NumericField):
    visible_from_root = False
    member_type = schema.Integer


class DecimalField(NumericField):
    visible_from_root = False
    member_type = schema.Decimal


class DateField(Field):

    visible_from_root = False
    member_type = schema.Date

    field_edit_controls = [
        "cocktail.html.DatePicker",
        "cocktail.html.CompoundDateSelector"
    ]


class TimeField(Field):
    visible_from_root = False
    member_type = schema.Time


class DateTimeField(Field):
    visible_from_root = False
    member_type = schema.DateTime




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models import VideoPlayerSettings
from woost.extensions.tv3alacarta.tv3alacartavideo import TV3ALaCartaVideo

def player_initialization(player, settings, video):
    player.video = video
    player.width = settings.width
    player.height = settings.height
    player.autoplay = settings.autoplay
    player.show_player_controls = settings.show_player_controls

VideoPlayerSettings.player_initialization[TV3ALaCartaVideo] = player_initialization




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
try:
    from collections import Counter
except ImportError:
    from backport_collections import Counter

from PIL import Image, ImageEnhance, ImageFilter, ImageOps, ImageChops
from cocktail.events import event_handler
from cocktail import schema
from cocktail.translations import translations
from woost.models.item import Item
from woost.models.file import File
from woost.models.rendering.request import clear_image_cache_after_commit


class ImageEffect(Item):

    instantiable = False
    visible_from_root = False

    image_factory = schema.Reference(
        type = "woost.models.rendering.ImageFactory",
        bidirectional = True,
        visible = False
    )

    @event_handler
    def handle_changed(cls, e):
        if e.source.is_inserted and e.source.image_factory:
            clear_image_cache_after_commit(factory = e.source.image_factory)


class ImageSize(schema.String):

    @classmethod
    def resolve_size(cls, value, size):

        if isinstance(value, basestring):
            if value.endswith("%"):
                value = float(value.rstrip("%")) / 100
            elif "." in value:
                value = float(value)
            else:
                value = int(value)

        if isinstance(value, float):
            number = int(size * value)
        if value < 0:
            value += size

        return value


class HorizontalAlignment(schema.String):

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("enumeration", ["left", "center", "right"])
        schema.String.__init__(self, *args, **kwargs)

    def translate_value(self, value, language = None, **kwargs):
        if not value:
            return ""
        else:
            return translations(
                "woost.models.rendering.horizontal_alignment=" + value,
                language,
                kwargs
            )


class VerticalAlignment(schema.String):

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("enumeration", ["top", "center", "bottom"])
        schema.String.__init__(self, *args, **kwargs)

    def translate_value(self, value, language = None, **kwargs):
        if not value:
            return ""
        else:
            return translations(
                "woost.models.rendering.vertical_alignment=" + value,
                language,
                kwargs
            )


def resolve_color(value):

    if isinstance(value, basestring):

        value = value.lstrip("#")

        if len(value) in (3, 4):
            value = tuple(int(d * 2, 16) for d in value)
        elif len(value) == 6:
            value = (
                int(value[0:2], 16),
                int(value[2:4], 16),
                int(value[4:6], 16)
            )
        elif len(value) == 8:
            value = (
                int(value[0:2], 16),
                int(value[2:4], 16),
                int(value[4:6], 16),
                int(value[6:8], 16)
            )
        else:
            raise ValueError("Invalid color: " + value)

    return value


class Thumbnail(ImageEffect):

    instantiable = True

    members_order = [
        "width",
        "height",
        "upscale"
    ]

    width = ImageSize()

    height = ImageSize()

    upscale = schema.Boolean(
        required = True,
        default = False
    )

    filter = Image.ANTIALIAS

    def apply(self, image):

        source_width, source_height = image.size

        width = self.width
        height = self.height

        if width is not None:
            width = ImageSize.resolve_size(width, source_width)

        if height is not None:
            height = ImageSize.resolve_size(height, source_height)

        if width is None:
            width = int(source_width * (float(height) / source_height))

        if height is None:
            height = int(source_height * (float(width) / source_width))

        if self.upscale and (width > source_width or height > source_height):
            resize_ratio = min(
                float(width) / source_width,
                float(height) / source_height
            )
            width = int(source_width * resize_ratio)
            height = int(source_height * resize_ratio)
            image = image.resize((width, height), self.filter)
        else:
            image.thumbnail((width, height), self.filter)

        return image


class Crop(ImageEffect):

    instantiable = True

    members_order = [
        "left",
        "top",
        "right",
        "bottom"
    ]

    left = ImageSize()

    top = ImageSize()

    right = ImageSize()

    bottom = ImageSize()

    def apply(self, image):
        width, height = image.size
        return image.crop((
            ImageSize.resolve_size(self.left, width),
            ImageSize.resolve_size(self.top, height),
            ImageSize.resolve_size(self.right, width),
            ImageSize.resolve_size(self.bottom, height)
        ))


class AutoCrop(ImageEffect):

    # Adapted from the following recipe:
    # http://megasnippets.com/en/source-codes/python/automatically_crop_image

    instantiable = True

    def apply(self, image):

        bbox = None

        # If the image has an alpha (tranparency) layer, we use it to crop the
        # image. Otherwise, we look at the pixels around the image (top, left,
        # bottom and right) and use the most used color as the color to crop.

        # Transparent images: use the alpha band to crop the image
        if 'A' in image.getbands():
            bbox = image.split()[list(image.getbands()).index('A')].getbbox()

        # Images with no alpha band: use the dominant color for cropping
        else:
            if image.mode != "RGB":
                image = image.convert("RGB")

            # Since .getbbox() always crops the black color, we need to
            # substract the "background" color from our image.
            background = get_dominant_edge_color(image)
            bg = Image.new("RGB", image.size, background)
            diff = ImageChops.difference(image, bg)
            bbox = diff.getbbox()

        if bbox:
            image = image.crop(bbox)

        return image


def get_dominant_edge_color(image):

    if image.mode != 'RGB':
        image = image.convert("RGB")

    # Get pixels from the edges of the image:
    width, height = image.size
    pixels = (
        # Left
        image.crop((0, 1, 1, height - 1)).tostring()

        # Right
        + image.crop((width - 1, 1, width, height - 1)).tostring()

        # Top
        + image.crop((0, 0, width, 1)).tostring()

        # Bottom
        + image.crop((0, height - 1, width, height)).tostring()
    )

    # Count the number of appearences of each color in the edge
    count = Counter()

    for i in range(0, len(pixels), 3):
        color = pixels[i] + pixels[i+1] + pixels[i+2]
        count[color] += 1

    dominant_color = count.most_common(1)[0][0]
    return tuple(map(ord, dominant_color))


class Fill(ImageEffect):

    instantiable = True

    members_order = [
        "width",
        "height",
        "upscale",
        "horizontal_alignment",
        "vertical_alignment",
        "preserve_vertical_images"
    ]

    width = ImageSize()

    height = ImageSize()

    upscale = schema.Boolean(
        required = True,
        default = False
    )

    horizontal_alignment = HorizontalAlignment(
        required = True,
        default = "center"
    )

    vertical_alignment = VerticalAlignment(
        required = True,
        default = "center"
    )

    preserve_vertical_images = schema.Boolean(
        required = True,
        default = False
    )

    filter = Image.ANTIALIAS

    def apply(self, image):

        source_width, source_height = image.size
        source_ratio = float(source_width) / source_height

        if self.preserve_vertical_images and source_ratio < 1:

            target_width = ImageSize.resolve_size(self.width, source_width)
            target_height = ImageSize.resolve_size(self.height, source_height)

            if (
                self.upscale
                and (
                    target_width > source_width
                    or target_height > source_height
                )
            ):
                resize_ratio = min(
                    float(target_width) / source_width,
                    float(target_height) / source_height
                )
                target_width = int(source_width * resize_ratio)
                target_height = int(source_height * resize_ratio)
                image = image.resize((target_width, target_height), self.filter)
            else:
                image.thumbnail(
                    (target_width, target_height),
                    self.filter
                )

            return image

        width = ImageSize.resolve_size(self.width, source_width)
        height = ImageSize.resolve_size(self.height, source_height)

        if width == source_width and height == source_height:
            return image

        resize_ratio = max(
            float(width) / source_width,
            float(height) / source_height
        )
        target_width = int(source_width * resize_ratio)
        target_height = int(source_height * resize_ratio)

        if self.upscale and (
            target_width > source_width
            or target_height > source_height
        ):
            image = image.resize((target_width, target_height), self.filter)
        else:
            width = min(source_width, width)
            height = min(source_height, height)
            target_width = min(source_width, target_width)
            target_height = min(source_height, target_height)
            image.thumbnail((target_width, target_height), self.filter)

        if self.horizontal_alignment == "center":
            offset_x = (target_width - width) / 2
        elif self.horizontal_alignment == "left":
            offset_x = 0
        elif self.horizontal_alignment == "right":
            offset_x = target_width - width
        else:
            raise ValueError(
                "Fill.horizontal_alignment = %s not implemented"
                % self.horizontal_alignment
            )

        if self.vertical_alignment == "center":
            offset_y = (target_height - height) / 2
        elif self.vertical_alignment == "top":
            offset_y = 0
        elif self.vertical_alignment == "bottom":
            offset_y = target_height - height
        else:
            raise ValueError(
                "Fill.vertical_alignment = %s not implemented"
                % self.horizontal_alignment
            )

        thumbnail_width, thumbnail_height = image.size

        image = image.crop((
            min(offset_x, thumbnail_width),
            min(offset_y, thumbnail_height),
            min(width + offset_x, thumbnail_width),
            min(height + offset_y, thumbnail_height)
        ))

        return image


class Rotate(ImageEffect):

    instantiable = True

    angle = schema.Integer(
        required = True
    )

    filter = Image.BICUBIC

    def apply(self, image):
        if image.mode != "RGBA":
            image = image.convert("RGBA")
        return image.rotate(self.angle, self.filter, True)


class Color(ImageEffect):

    instantiable = True

    level = schema.Float(
        required = True
    )

    def apply(self, image):
        return ImageEnhance.Color(image).enhance(self.level)


class Brightness(ImageEffect):

    instantiable = True

    level = schema.Float(
        required = True
    )

    def apply(self, image):
        return ImageEnhance.Brightness(image).enhance(self.level)


class Contrast(ImageEffect):

    instantiable = True

    level = schema.Float(
        required = True
    )

    def apply(self, image):
        return ImageEnhance.Contrast(image).enhance(self.level)


class Sharpness(ImageEffect):

    instantiable = True

    level = schema.Float(
        required = True
    )

    def apply(self, image):
        return ImageEnhance.Sharpness(image).enhance(self.level)


class Frame(ImageEffect):

    instantiable = True

    members_order = [
        "edge_width",
        "edge_color",
        "vertical_padding",
        "horizontal_padding",
        "background"
    ]

    edge_width = schema.Integer(
        required = True,
        min = 1,
        default = 1
    )

    edge_color = schema.Color(
        required = True,
        default = "#000000"
    )

    vertical_padding = ImageSize(
        required = True,
        default = "0"
    )

    horizontal_padding = ImageSize(
        required = True,
        default = "0"
    )

    background = schema.Color(
        required = True,
        default = "#000000"
    )

    def apply(self, image):

        edge_color = resolve_color(self.edge_color)
        background = resolve_color(self.background)

        # Create the canvas
        width, height = image.size
        vertical_padding = ImageSize.resolve_size(self.vertical_padding, height)
        vertical_offset = self.edge_width + vertical_padding
        horizontal_padding = ImageSize.resolve_size(self.horizontal_padding, width)
        horizontal_offset = self.edge_width + horizontal_padding

        canvas = Image.new("RGBA", (
            width + horizontal_offset * 2,
            height + vertical_offset * 2
        ))

        # Paint the border
        edge = self.edge_width
        if edge:
            canvas.paste(edge_color, None)

        # Paint the padding color
        canvas.paste(
            background,
            (edge,
             edge,
             edge + width + horizontal_padding * 2,
             edge + height + vertical_padding * 2)
        )

        # Paste the original image over the frame
        canvas.paste(
            image,
            (horizontal_offset, vertical_offset),
            image if image.mode in ("1", "L", "RGBA") else None
        )

        return canvas


class Shadow(ImageEffect):

    instantiable = True

    offset = schema.Integer(
        required = True,
        default = 5
    )

    padding = schema.Integer(
        required = True,
        default = 8
    )

    color = schema.Color(
        required = True,
        default = "#444444"
    )

    iterations = 3

    def apply(self, image):

        # Create the backdrop image -- a box in the background colour with a
        # shadow on it.
        total_width = image.size[0] + abs(self.offset) + 2 * self.padding
        total_height = image.size[1] + abs(self.offset) + 2 * self.padding
        back = Image.new("RGBA", (total_width, total_height))

        # Place the shadow, taking into account the offset from the image
        shadow_left = self.padding + max(self.offset, 0)
        shadow_top = self.padding + max(self.offset, 0)
        color = resolve_color(self.color)
        back.paste(self.color, [
            shadow_left,
            shadow_top,
            shadow_left + image.size[0],
            shadow_top + image.size[1]
        ])

        # Apply the filter to blur the edges of the shadow.  Since a small
        # kernel is used, the filter must be applied repeatedly to get a
        # decent blur.
        for n in range(self.iterations):
            back = back.filter(ImageFilter.BLUR)

        # Paste the input image onto the shadow backdrop
        image_left = self.padding - min(self.offset, 0)
        image_top = self.padding - min(self.offset, 0)
        back.paste(
            image,
            (image_left, image_top),
            image if image.mode in ("1", "L", "RGBA") else None
        )

        return back


class ReducedOpacity(ImageEffect):
    """An image effect that reduces the opacity of the given image."""

    instantiable = True

    level = schema.Float(
        required = True,
        default = 0.5,
        min = 0.0,
        max = 1.0
    )

    def apply(self, image):
        return reduce_opacity(image, self.level)


def reduce_opacity(image, opacity):

    if opacity < 0 or opacity > 1:
        raise ValueError(
            "reduce_opacity() expects a floating point number "
            "between 0 and 1: got %s instead"
            % opacity
        )

    if image.mode != 'RGBA':
        image = image.convert('RGBA')
    else:
        image = image.copy()

    alpha = image.split()[3]
    alpha = ImageEnhance.Brightness(alpha).enhance(opacity)
    image.putalpha(alpha)
    return image


class Fade(ImageEffect):

    instantiable = True

    members_order = [
        "top_width",
        "right_width",
        "bottom_width",
        "left_width"
    ]

    top_width = schema.Integer(
        required = True,
        default = 0,
        min = 0
    )

    right_width = schema.Integer(
        required = True,
        default = 0,
        min = 0
    )

    bottom_width = schema.Integer(
        required = True,
        default = 0,
        min = 0
    )

    left_width = schema.Integer(
        required = True,
        default = 0,
        min = 0
    )

    edge_opacity = schema.Integer(
        required = True,
        default = 0,
        min = 0,
        max = 255
    )

    inner_opacity = schema.Integer(
        required = True,
        default = 255,
        min = 0,
        max = 255
    )

    def apply(self, image):

        if image.mode != "RGBA":
            image = image.convert("RGBA")

        pixels = image.load()
        w, h = image.size

        tw = self.top_width
        rw = self.right_width
        bw = self.bottom_width
        lw = self.left_width

        inner_opacity = self.inner_opacity
        edge_opacity = self.edge_opacity
        opacity_delta = self.inner_opacity - edge_opacity

        for x in xrange(w):
            for y in xrange(h):

                if x < lw:
                    hor_opacity = float(x) / lw
                elif x > w - rw:
                    hor_opacity = float(rw - (x - (w - rw))) / rw
                else:
                    hor_opacity = None

                if y < tw:
                    ver_opacity = float(y) / tw
                elif y > h - bw:
                    ver_opacity = float(bw - (y - (h - bw))) / bw
                else:
                    ver_opacity = None

                if hor_opacity is None and ver_opacity is None:
                    opacity = inner_opacity
                else:
                    if hor_opacity is None:
                        step = ver_opacity
                    elif ver_opacity is None:
                        step = hor_opacity
                    else:
                        step = min(ver_opacity, hor_opacity)

                    opacity = int(edge_opacity + opacity_delta * step)

                p = pixels[x, y]
                pixels[x, y] = (p[0], p[1], p[2], opacity)

        return image


class Watermark(ImageEffect):

    instantiable = True

    mark = schema.Reference(
        type = File,
        required = True,
        related_end = schema.Collection()
    )

    opacity = schema.Float(
        required = True,
        min = 0.0,
        max = 1.0,
        default = 1.0
    )

    placement = schema.String(
        required = True,
        default = "middle",
        enumeration = ["middle", "scale", "tile"]
    )

    def apply(self, image):

        mark_image = Image.open(self.mark.file_path)

        if self.opacity < 1:
            mark_image = reduce_opacity(mark_image, opacity)

        if image.mode != 'RGBA':
            image = image.convert('RGBA')

        # Create a transparent layer the size of the image and draw the
        # watermark in that layer.
        layer = Image.new('RGBA', image.size, (0,0,0,0))

        if self.placement == 'tile':
            for y in range(0, image.size[1], mark_image.size[1]):
                for x in range(0, image.size[0], mark_image.size[0]):
                    layer.paste(mark_image, (x, y), mark_image)
        elif self.placement == 'scale':
            # scale, but preserve the aspect ratio
            ratio = min(
                float(image.size[0]) / mark_image.size[0],
                float(image.size[1]) / mark_image.size[1]
            )
            w = int(mark_image.size[0] * ratio)
            h = int(mark_image.size[1] * ratio)
            mark_image = mark_image.resize((w, h))
            layer.paste(
                mark_image,
                ((image.size[0] - w) / 2, (image.size[1] - h) / 2),
                mark_image
            )
        elif self.placement == 'middle':
            layer.paste(
                mark_image,
                (
                    (image.size[0] - mark_image.size[0]) / 2,
                    (image.size[1] - mark_image.size[1]) / 2
                ),
                mark_image
            )
        else:
            raise ValueError(
                "Must specify position parameter [tile,scale,middle]."
            )

        # composite the watermark with the layer
        return Image.composite(layer, image, layer)


class Flip(ImageEffect):

    instantiable = True

    axis = schema.String(
        required = True,
        edit_control = "cocktail.html.RadioSelector",
        enumeration = ["horizontal", "vertical"]
    )

    def apply(self, image):

        if self.axis == "horizontal":
            axis = Image.FLIP_LEFT_RIGHT
        elif self.axis == "vertical":
            axis = Image.FLIP_TOP_BOTTOM
        else:
            raise TypeError(
                "Flip.axis can either be 'horizontal' "
                "or 'vertical'; got %s." % self.direction
            )

        return image.transpose(axis)


class Align(ImageEffect):

    instantiable = True

    members_order = [
        "width",
        "height",
        "horizontal_alignment",
        "vertical_alignment",
        "background"
    ]

    width = ImageSize()

    height = ImageSize()

    horizontal_alignment = HorizontalAlignment(
        required = True,
        default = "center"
    )

    vertical_alignment = VerticalAlignment(
        required = True,
        default = "center"
    )

    background = schema.Color()

    def apply(self, image):

        source_width, source_height = image.size
        width = ImageSize.resolve_size(self.width, source_width)
        height = ImageSize.resolve_size(self.height, source_height)

        needs_halign = (source_width < width)
        needs_valign = (source_height < height)

        if needs_halign or needs_valign:

            if self.background:
                background = resolve_color(self.background)
            elif image.mode == "RGBA":
                background = (0,0,0,0)
            else:
                background = (255,255,255)

            copy = Image.new(
                "RGBA" if background and len(background) == 4 else "RGB",
                (width, height),
                background
            )

            x = 0
            y = 0

            if needs_halign:
                if self.horizontal_alignment == "left":
                    pass
                elif self.horizontal_alignment == "center":
                    x = width / 2 - source_width / 2
                elif self.horizontal_alignment == "right":
                    x = width - source_width
                else:
                    raise ValueError(
                        "Align.horizontal_alignment expects 'left', 'center' "
                        "or 'right', got %s" % self.horizontal_alignment
                    )

            if needs_valign:
                if self.vertical_alignment == "top":
                    pass
                elif self.vertical_alignment == "center":
                    y = height / 2 - source_height / 2
                elif self.vertical_alignment == "bottom":
                    y = height - source_height
                else:
                    raise ValueError(
                        "Align.horizontal_alignment expects 'top', 'center' "
                        "or 'bottom', got %s" % self.vertical_alignment
                    )

            copy.paste(image, (x, y), image if image.mode == "RGBA" else None)
            return copy

        return image


class Grayscale(ImageEffect):

    instantiable = True

    def apply(self, image):
        return ImageOps.grayscale(image)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Pepe Osca <pepe.osca@whads.com>
"""
from cocktail import schema
from woost.models import Configuration
from .identityprovider import IdentityProvider

Configuration.add_member(
    schema.Collection("identity_providers",
        items = schema.Reference(type = IdentityProvider),
        related_end = schema.Reference(),
        integral = True,
        member_group = "services"
    )
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
import re
from cocktail import schema
from cocktail.controllers import Location
from woost.models.publishable import Publishable

video_id_expr = re.compile(r"www\.tv3\.cat/videos/(\d+)")

def extract_video_id(string):

    if string:
        match = video_id_expr.search(string)
        if match:
            return match.group(1)

    return string


class TV3ALaCartaVideo(Publishable):

    instantiable = True
    type_group = "resource"
    uri_pattern = "http://www.tv3.cat/videos/%s"
    video_player = "woost.extensions.tv3alacarta.TV3ALaCartaPlayer"

    default_resource_type = "video"

    members_order = ["title", "video_id"]

    title = schema.String(
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True,
        spellcheck = True,
        member_group = "content"
    )

    video_id = schema.String(
        required = True,
        listed_by_default = False,
        normalization = extract_video_id,
        member_group = "content"
    )

    def get_uri(self,
        path = None,
        parameters = None,
        language = None,
        host = None,
        encode = True):
        return self.uri_pattern % self.video_id

    def is_internal_content(self, language = None):
        return False




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from simplejson import dumps
from cocktail.translations import translations, get_language
from cocktail.events import Event
from woost import app
from woost.models import (
    Extension,
    extension_translations,
    Configuration,
    Publishable
)

translations.define("GoogleAnalyticsExtension",
    ca = u"Google Analytics",
    es = u"Google Analytics",
    en = u"Google Analytics"
)

translations.define("GoogleAnalyticsExtension-plural",
    ca = u"Google Analytics",
    es = u"Google Analytics",
    en = u"Google Analytics"
)

translations.define("GoogleAnalyticsExtension.account",
    ca = u"Compte",
    es = u"Cuenta",
    en = u"Account"
)


class GoogleAnalyticsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Integra el lloc web amb Google Analytics.""",
            "ca"
        )
        self.set("description",
            u"""Integra el sitio web con Google Analytics.""",
            "es"
        )
        self.set("description",
            u"""Integrates the site with Google Analytics.""",
            "en"
        )

    def _load(self):

        from woost.extensions.googleanalytics import (
            strings,
            configuration,
            website,
            document,
            block,
            element,
            eventredirection,
            customdefinition
        )

        # Install an overlay to generate events for links pointing to
        # publishable elements
        from cocktail.html import templates
        templates.get_class("woost.extensions.googleanalytics.BaseViewOverlay")
        templates.get_class("woost.extensions.googleanalytics.LinkOverlay")
        templates.get_class("woost.extensions.googleanalytics.TextBlockViewOverlay")
        templates.get_class("woost.extensions.googleanalytics.LanguageSelectorOverlay")

        from cocktail.events import when
        from woost.controllers import CMSController

        @when(CMSController.producing_output)
        def handle_producing_output(e):
            publishable = e.output.get("publishable")
            if (
                publishable is None
                or getattr(publishable, "is_ga_tracking_enabled", lambda: True)()
            ):
                html = e.output.get("head_end_html", "")
                if html:
                    html += " "
                html += self.get_analytics_page_hit_script(publishable)
                e.output["head_end_html"] = html

    def _install(self):
        from woost.extensions.googleanalytics import installationstrings
        self._create_default_custom_definitions()

    def _create_default_custom_definitions(self):

        from .customdefinition import GoogleAnalyticsCustomDefinition

        Configuration.instance.google_analytics_custom_definitions.extend([
            self._create_asset(
                GoogleAnalyticsCustomDefinition,
                "default_custom_definitions.locale",
                title = extension_translations,
                identifier = "woost.locale",
                initialization =
                    "from cocktail.translations import get_language\n"
                    "value = get_language()"
            ),
            self._create_asset(
                GoogleAnalyticsCustomDefinition,
                "default_custom_definitions.roles",
                title = extension_translations,
                identifier = "woost.roles",
                initialization =
                    "from woost import app\n"
                    "value = set(app.user.iter_roles())"
            ),
            self._create_asset(
                GoogleAnalyticsCustomDefinition,
                "default_custom_definitions.path",
                title = extension_translations,
                identifier = "woost.path",
                initialization =
                    "value = reversed(list(publishable.ascend_tree(True)))"
            ),
            self._create_asset(
                GoogleAnalyticsCustomDefinition,
                "default_custom_definitions.publishable",
                title = extension_translations,
                identifier = "woost.publishable",
                initialization =
                    "value = publishable"
            ),
            self._create_asset(
                GoogleAnalyticsCustomDefinition,
                "default_custom_definitions.type",
                title = extension_translations,
                identifier = "woost.type",
                initialization =
                    "from woost.models import Publishable\n"
                    "value = [\n"
                    "   cls\n"
                    "   for cls in publishable.__class__.__mro__\n"
                    "   if cls is not Publishable and issubclass(cls, Publishable)\n"
                    "]"
            )
        ])

    inclusion_code = {
        "async":
            """
            <script type="text/javascript">
              (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
              (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
              m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
              })(window,document,'script','//www.google-analytics.com/analytics.js','ga');
              %(create_tracker_command)s
              %(commands)s
            </script>
            """,
        "sync":
            """
            <script type="text/javascript">
              (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
              (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
              m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
              })(window,document,'script','//www.google-analytics.com/analytics.js','ga');
              %(create_tracker_command)s;
              ga(function () {
                  %(commands)s
              });
            </script>
            """,
    }

    declaring_tracker = Event()

    def get_analytics_script(self,
        publishable = None,
        commands = None,
        async = True
    ):
        from .utils import get_ga_custom_values

        config = Configuration.instance

        if publishable is None:
            publishable = app.publishable

        event = self.declaring_tracker(
            publishable = publishable,
            account = config.get_setting("google_analytics_account"),
            tracker_parameters = {},
            domain = config.get_setting("google_analytics_domain"),
            template = self.inclusion_code["async" if async else "sync"],
            values = get_ga_custom_values(publishable),
            commands = commands or []
        )

        if not event.account:
            return ""

        commands = event.commands
        parameters = {}

        if event.domain:
            event.tracker_parameters["cookieDomain"] = event.domain

        parameters["create_tracker_command"] = \
            self._serialize_commands([(
                "create",
                event.account,
                event.tracker_parameters
            )])

        if event.values:
            commands.insert(0, ("set", event.values))

        parameters["commands"] = self._serialize_commands(commands)
        return event.template % parameters

    def get_analytics_page_hit_script(self, publishable = None):
        return self.get_analytics_script(
            publishable = publishable,
            commands = [("send", "pageview")]
        )

    def _serialize_commands(self, commands):
        return "\n".join(
            "ga(%s);" % (", ".join(dumps(arg) for arg in cmd))
            for cmd in commands
        )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
from .page import (
    Page as StandardPage,
    PageTranslation as StandardPageTranslation
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# Configuration / Website
#------------------------------------------------------------------------------
for cls_name in ("Configuration", "Website"):
    translations.define(cls_name + ".services.google_analytics",
        ca = u"Google Analytics",
        es = u"Google Analytics",
        en = u"Google Analytics"
    )

    translations.define(cls_name + ".google_analytics_account",
        ca = u"Compte de Google Analytics",
        es = u"Cuenta de Google Analytics",
        en = u"Google Analytics account"
    )

    translations.define(cls_name + ".google_analytics_domain",
        ca = u"Domini per Google Analytics",
        es = u"Dominio para Google Analytics",
        en = u"Domain for Google Analytics"
    )

    translations.define(cls_name + ".google_analytics_language",
        ca = u"Idioma pels literals enviats a Google Analytics",
        es = u"Idioma para los literales enviados a Google Analytics",
        en = u"Language of text fragments sent to Google Analytics"
    )

    translations.define(cls_name + ".google_analytics_language-explanation",
        ca = u"Algunes funcionalitats, com ara la generació automàtica "
             u"d'esdeveniments pels clics a blocs de text i imatge, poden "
             u"inserir textos a les estadístiques de Google Analytics (per "
             u"exemple, el nom del bloc on s'ha fet el clic). Aquesta opció "
             u"determina l'idioma que s'hauria d'utilitzar per obtenir "
             u"aquests textos.",
        es = u"Algunas funcionalidades, como la generación automática de "
             u"eventos para los clics en bloques de texto e imagen, pueden "
             u"insertar textos en las estadísticas de Google Analytics (por "
             u"ejemplo, el nombre del bloque donde se ha hecho el clic). Esta "
             u"opción determina el idioma que debería utilizarse para obtener "
             u"estos textos.",
        en = u"Some features, such as the automatic generation of events for "
             u"clicks on text + image blocks, can insert text literals into "
             u"Google Analytics statistics (for example, the name of the "
             u"block that received the click). This option controls the "
             u"language that should be used to obtain those texts."
    )

    translations.define(cls_name + ".google_analytics_custom_definitions",
        ca = u"Definicions personalitzades",
        es = u"Definiciones personalizadas",
        en = u"Custom definitions"
    )

    translations.define(
        cls_name + ".google_analytics_custom_definitions-explanation",
        ca = u"Afegeix dimensions o mètriques personalitzades a les "
             u"estadístiques. Important: s'han de replicar les definicions "
             u"manualment a la configuració de la propietat de Google Analytics,"
             u"en el mateix ordre en que es defineixin aquí.",
        es = u"Añade dimensiones o métricas personalizadas a las estadísticas. "
             u"Importante: las definiciones deben replicarse manualmente en la "
             u"configuración de la propiedad de Google Analytics, en el mismo "
             u"orden en que se definan aquí.",
        en = u"Adds custom dimensions or metrics to the page hit / event "
             u"statistics. Important: all definitions must be manually replicated "
             u"on the configuration of the Google Analytics property, in the same "
             u"order they are defined here."
    )

translations.define("Document.ga_tracking_enabled",
    ca = u"Habilitar seguiment amb Google Analytics",
    es = u"Habilitar seguimiento con Google Analytics",
    en = u"Track with Google Analytics"
)

# Client redirections
#------------------------------------------------------------------------------
translations.define("woost.extensions.googleanalytics.redirection_title",
    ca = u"Obrint recurs",
    es = u"Abriendo recurso",
    en = u"Downloading resource"
)

translations.define("woost.extensions.googleanalytics.redirection_body",
    ca = lambda url:
        u"S'està obrint el recurs sol·licitat. Si no s'obre automàticament, "
        u"si us plau faci clic <a href='%s'>aquí</a>."
        % url,
    es = lambda url:
        u"Se está abriendo el recurso solicitado. Si no se abre "
        u"automáticamente, por favor haga clic <a href='%s'>aquí</a>."
        % url,
    en = lambda url:
        u"The page or file you requested is being downloaded. If the download "
        u"doesn't start automatically, please click <a href='%s'>here</a>."
        % url
)

# GoogleAnalyticsCustomDefinition
#------------------------------------------------------------------------------
translations.define("GoogleAnalyticsCustomDefinition",
    ca = u"Definició personalitzada de Google Analytics",
    es = u"Definición personalizada de Google Analytics",
    en = u"Custom definition for Google Analytics"
)

translations.define("GoogleAnalyticsCustomDefinition-plural",
    ca = u"Definicions personalitzada de Google Analytics",
    es = u"Definiciones personalizadas de Google Analytics",
    en = u"Custom definitions for Google Analytics"
)

translations.define("GoogleAnalyticsCustomDefinition.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("GoogleAnalyticsCustomDefinition.identifier",
    ca = u"Identificador",
    es = u"Identificador",
    en = u"Identifier"
)

translations.define("GoogleAnalyticsCustomDefinition.definition_type",
    ca = u"Tipus de definició",
    es = u"Tipo de definición",
    en = u"Definition type"
)

translations.define(
    "GoogleAnalyticsCustomDefinition.definition_type=dimension",
    ca = u"Dimensió",
    es = u"Dimensión",
    en = u"Dimension"
)

translations.define(
    "GoogleAnalyticsCustomDefinition.definition_type=metric",
    ca = u"Mètrica",
    es = u"Métrica",
    en = u"Metric"
)

translations.define("GoogleAnalyticsCustomDefinition.enabled",
    ca = u"Activa",
    es = u"Activa",
    en = u"Enabled"
)

translations.define("GoogleAnalyticsCustomDefinition.content_types",
    ca = u"Tipus d'element",
    es = u"Tipos de elemento",
    en = u"Content types"
)

translations.define("GoogleAnalyticsCustomDefinition.initialization",
    ca = u"Inicialització",
    es = u"Inicialización",
    en = u"Initialization"
)

# LanguageSelectorOverlay
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.googleanalytics."
    "LanguageSelectorOverlay.ga_event_category",
    ca = u"selector-idiomes",
    es = u"selector-de-idiomas",
    en = u"language-selector"
)

translations.define(
    "woost.extensions.googleanalytics."
    "LanguageSelectorOverlay.ga_event_action",
    ca = u"seleccionar-idioma",
    es = u"seleccionar-idioma",
    en = u"select-language"
)




# -*- coding: utf-8 -*-
u"""

.. moduleauthor:: Pepe Osca <pepe.osca@whads.com>
"""
from urllib import urlencode

from cocktail import schema
from cocktail.controllers import Location
from woost.extensions.identity.identityprovider import IdentityProvider


class FacebookIdentityProvider(IdentityProvider):
    provider_name = "Facebook"
    user_identifier = "facebook_user_id"

    members_order = [
        "client_id",
        "client_secret",
        "scope"
    ]

    client_id = schema.String(
        required=True,
        text_search=False
    )

    client_secret = schema.String(
        required=True,
        text_search=False
    )

    scope = schema.Collection(
        min=1,
        default=schema.DynamicDefault(lambda: ["public_profile", "email"]),
        items=schema.String()
    )

    def get_auth_url(self, target_url = None):
        return (
            "/facebook_oauth/%d/step1?%s" % (
                self.id,
                urlencode({
                    "target_url":
                        target_url
                        or unicode(
                            Location.get_current(relative=False)).encode(
                            "utf-8")
                })
            )
        )



#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from cocktail.controllers import get_parameter
from woost import app
from .transition import TranslationWorkflowTransition
from .transitionpermission import TranslationWorkflowTransitionPermission
from .utils import iter_changeset_translation_requests

def apply_default_transition(e):

    if e.change is None:
        return

    if not get_parameter(
        schema.Boolean("translation_workflow_auto_transition")
    ):
        return

    user = app.user

    if user is None:
        return

    transition = None
    condition = None

    for role in user.iter_roles():
        if role.translation_workflow_default_transition:
            transition = role.translation_workflow_default_transition
            if role.translation_workflow_default_transition_condition:
                condition = compile(
                    role.translation_workflow_default_transition_condition,
                    "%s #%s.translation_workflow_default_transition_condition" % (
                        role.__class__.__name__,
                        role.id
                    ),
                    "exec"
                )
            break

    if transition is None:
        return

    for request in list(
        iter_changeset_translation_requests(
            e.change.changeset,
            include_unchanged = True
        )
    ):
        if (
            (
                not transition.source_states
                or request.state in transition.source_states
            )
            and user.has_permission(
                TranslationWorkflowTransitionPermission,
                transition = transition,
                translation_request = request
            )
        ):
            if condition is not None:
                context = {
                    "request": request,
                    "transition": transition,
                    "applies": True
                }
                exec condition in context
                if not context["applies"]:
                    continue
            transition.execute(request)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from woost.models import LocaleMember
from woost.views.uigeneration import backoffice_edit_control


class TranslationWorkflowPath(schema.Tuple):

    def __init__(self, *args, **kwargs):
        kwargs["items"] = (
            LocaleMember("source_language",
                required = True,
            ),
            LocaleMember("target_language",
                required = True
            )
        )
        kwargs["request_value_separator"] = ":"
        schema.Tuple.__init__(self, *args, **kwargs)

    def translate_value(self, value, language = None, **kwargs):
        if value:
            try:
                return translations(
                    "woost.extensions.translationworkflow.translation_path",
                    source_language = value[0],
                    target_language = value[1],
                    language = language,
                    **kwargs
                )
            except Exception, error:
                pass

        return schema.Tuple.translate_value(
            self,
            value,
            language = language,
            **kwargs
        )


backoffice_edit_control.set_member_type_display(
    TranslationWorkflowPath,
    "woost.extensions.translationworkflow.TranslationWorkflowPathEditor"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# Installation
#------------------------------------------------------------------------------
translations.define("woost.extensions.newsletters.newsletter_controller.title",
    ca = u"Controlador de butlletins",
    es = u"Controlador de boletines",
    en = u"Newsletter controller"
)

translations.define("woost.extensions.newsletters.newsletter_template.title",
    ca = u"Plantilla de butlletí",
    es = u"Plantilla de boletín",
    en = u"Newsletter template"
)

translations.define("woost.extensions.newsletters.image_factories.third.title",
    ca = u"Butlletí - 33%",
    es = u"Boletín - 33%",
    en = u"Newsletter - 33%"
)

translations.define("woost.extensions.newsletters.image_factories.half.title",
    ca = u"Butlletí - 50%",
    es = u"Boletín - 50%",
    en = u"Newsletter - 50%"
)

translations.define(
    "woost.extensions.newsletters.image_factories.full.title",
    ca = u"Butlletí - 100%",
    es = u"Boletín - 100%",
    en = u"Newsletter - 100%"
)

# ImageFactory
#------------------------------------------------------------------------------
translations.define("ImageFactory.applicable_to_newsletters",
    ca = u"Aplicable a butlletins de notícies",
    es = u"Aplicable a boletines de noticias",
    en = u"Applicable to newsletters"
)

# Newsletter
#------------------------------------------------------------------------------
translations.define("Newsletter",
    ca = u"Butlletí de notícies",
    es = u"Boletín de noticias",
    en = u"Newsletter"
)

translations.define("Newsletter-plural",
    ca = u"Butlletins de notícies",
    es = u"Boletines de noticias",
    en = u"Newsletters"
)

translations.define("Newsletter.blocks",
    ca = u"Blocs",
    es = u"Bloques",
    en = u"Blocks"
)

# NewsletterContent
#------------------------------------------------------------------------------
translations.define("NewsletterContent",
    ca = u"Contingut",
    es = u"Contenido",
    en = u"Content"
)

translations.define("NewsletterContent-plural",
    ca = u"Continguts",
    es = u"Contenidos",
    en = u"Content"
)

translations.define("NewsletterContent.view_class",
    ca = u"Aparença",
    es = u"Apariencia",
    en = u"Appearence"
)

translations.define(
    "NewsletterContent.view_class"
    "=woost.extensions.newsletters.NewsletterContentView",
    ca = u"Estàndard",
    es = u"Estándar",
    en = u"Standard"
)

translations.define("NewsletterContent.text",
    ca = u"Text",
    es = u"Texto",
    en = u"Text"
)

translations.define("NewsletterContent.image",
    ca = u"Imatge",
    es = u"Imagen",
    en = u"Image"
)

translations.define("NewsletterContent.image_alignment",
    ca = u"Alineament",
    es = u"Alineamiento",
    en = u"Alignment"
)

translations.define("NewsletterContent.image_alignment=image_left",
    ca = u"Imatge a l'esquerra",
    es = u"Imagen a la izquierda",
    en = u"Image left"
)

translations.define("NewsletterContent.image_alignment=image_right",
    ca = u"Imatge a la dreta",
    es = u"Imagen a la derecha",
    en = u"Image right"
)

translations.define("NewsletterContent.image_alignment=image_top",
    ca = u"Imatge a dalt",
    es = u"Imagen arriba",
    en = u"Image top"
)

translations.define("NewsletterContent.image_factory",
    ca = u"Processat d'imatge",
    es = u"Procesado de imagen",
    en = u"Image processing"
)

translations.define("NewsletterContent.link",
    ca = u"Enllaç",
    es = u"Enlace",
    en = u"Link"
)

# NewsletterBox
#------------------------------------------------------------------------------
translations.define("NewsletterBox",
    ca = u"Caixa",
    es = u"Caja",
    en = u"Box"
)

translations.define("NewsletterBox-plural",
    ca = u"Caixes",
    es = u"Cajas",
    en = u"Boxes"
)

translations.define("NewsletterBox.view_class",
    ca = u"Aparença",
    es = u"Apariencia",
    en = u"Appearence"
)

translations.define(
    "NewsletterBox.view_class"
    "=woost.extensions.newsletters.NewsletterBoxView",
    ca = u"Estàndard",
    es = u"Estándar",
    en = u"Standard"
)

translations.define("NewsletterBox.blocks",
    ca = u"Blocs",
    es = u"Bloques",
    en = u"Blocks"
)

# NewsletterListing
#------------------------------------------------------------------------------
translations.define("NewsletterListing",
    ca = u"Llistat",
    es = u"Listado",
    en = u"Listing"
)

translations.define("NewsletterListing-plural",
    ca = u"Llistats",
    es = u"Listados",
    en = u"Listings"
)

translations.define("NewsletterListing.view_class",
    ca = u"Aparença",
    es = u"Apariencia",
    en = u"Appearence"
)

translations.define(
    "NewsletterListing.view_class"
    "=woost.extensions.newsletters.NewsletterListingTextOnlyView",
    ca = u"Només text",
    es = u"Solo texto",
    en = u"Text only"
)

translations.define(
    "NewsletterListing.view_class"
    "=woost.extensions.newsletters.NewsletterListingTextAndIconView",
    ca = u"Text i icona",
    es = u"Texto e icono",
    en = u"Text and icon"
)

translations.define(
    "NewsletterListing.view_class"
    "=woost.extensions.newsletters.NewsletterListingSummaryView",
    ca = u"Resum",
    es = u"Resumen",
    en = u"Summary"
)

translations.define("NewsletterListing.listed_items",
    ca = u"Elements a llistar",
    es = u"Elementos a listar",
    en = u"Listed items"
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from datetime import date
from cocktail import schema
from .document import Document
from .file import File
from .slot import Slot


class News(Document):

    members_order = [
        "news_date",
        "image",
        "summary",
        "blocks"
    ]

    news_date = schema.Date(
        required = True,
        indexed = True,
        default = schema.DynamicDefault(date.today),
        member_group = "content"
    )

    image = schema.Reference(
        type = File,
        related_end = schema.Collection(),
        relation_constraints = {"resource_type": "image"},
        member_group = "content"
    )

    summary = schema.HTML(
        listed_by_default = False,
        translated = True,
        member_group = "content"
    )

    blocks = Slot()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import cherrypy
from cocktail import schema
from cocktail.events import event_handler
from cocktail.html import templates
from .publishable import Publishable
from .controller import Controller
from .metatags import MetaTags


class Document(Publishable):

    instantiable = True
    type_group = "document"
    default_per_language_publication = True

    groups_order = list(Publishable.groups_order) + [
        "meta",
        "robots"
    ]

    members_order = (
        "title",
        "inner_title",
        "template",
        "meta_tags",
        "description",
        "keywords",
        "children",
        "redirection_mode",
        "redirection_target",
        "redirection_method",
        "robots_should_index",
        "robots_should_follow"
    )

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(qname = "woost.document_controller")
    )

    title = schema.String(
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True,
        required = True,
        spellcheck = True,
        member_group = "content"
    )

    inner_title = schema.String(
        translated = True,
        listed_by_default = False,
        spellcheck = True,
        member_group = "content"
    )

    meta_tags = MetaTags(
        member_group = "meta"
    )

    description = schema.String(
        translated = True,
        listed_by_default = False,
        spellcheck = True,
        edit_control = "cocktail.html.TextArea",
        member_group = "meta"
    )

    keywords = schema.String(
        translated = True,
        listed_by_default = False,
        spellcheck = True,
        edit_control = "cocktail.html.TextArea",
        member_group = "meta"
    )

    template = schema.Reference(
        type = "woost.models.Template",
        bidirectional = True,
        listed_by_default = False,
        after_member = "controller",
        member_group = "presentation.behavior"
    )

    children = schema.Collection(
        items = "woost.models.Publishable",
        bidirectional = True,
        related_key = "parent",
        cascade_delete = True,
        after_member = "parent",
        member_group = "navigation"
    )

    redirection_mode = schema.String(
        enumeration = ["first_child", "custom_target"],
        listed_by_default = False,
        text_search = False,
        member_group = "navigation"
    )

    redirection_target = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        required = redirection_mode.equal("custom_target"),
        listed_by_default = False,
        member_group = "navigation"
    )

    redirection_method = schema.String(
        required = True,
        default = "temp",
        enumeration = ["temp", "perm", "client"],
        listed_by_default = False,
        text_search = False,
        member_group = "navigation"
    )

    robots_should_index = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "meta.robots"
    )

    robots_should_follow = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "meta.robots"
    )

    def _update_path(self, parent, path):

        Publishable._update_path(self, parent, path)

        if self.children:
            for child in self.children:
                child._update_path(self, child.path)

    def descend_tree(self, include_self = False):

        if include_self:
            yield self

        if self.children:
            for child in self.children:
                for descendant in child.descend_tree(True):
                    yield descendant

    def render(self, **values):
        """Renders the document using its template."""
        if self.template is None:
            raise ValueError("Can't render a document without a template")

        values["publishable"] = self

        view = templates.new(self.template.identifier)
        for key, value in values.iteritems():
            setattr(view, key, value)

        return view.render_page()

    def find_redirection_target(self):
        mode = self.redirection_mode

        if mode == "first_child":
            return self.find_first_child_redirection_target()

        elif mode == "custom_target":
            return self.redirection_target

    def find_first_child_redirection_target(self):
        for child in self.children:
            if child.is_accessible():
                if isinstance(child, Document):
                    return child.find_redirection_target() or child
                else:
                    return child

    def first_child_redirection(self):
        child = self.find_first_child_redirection_target()
        if child is not None:
            raise cherrypy.HTTPRedirect(child.get_uri())

    @event_handler
    def handle_related(cls, event):
        if event.member is cls.websites:
            for child in event.source.children:
                child.websites = set(event.source.websites)

    @event_handler
    def handle_unrelated(cls, event):
        if not event.source.is_deleted:
            if event.member is cls.websites:
                for child in event.source.children:
                    child.websites = set(event.source.websites)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail.events import event_handler
from cocktail import schema
from cocktail.controllers import request_property, get_parameter
from woost.models import Item
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController
from woost.models import Block
from woost.controllers.backoffice.editstack import EditBlocksNode


class EditBlocksController(BaseBackOfficeController):

    view_class = "woost.views.EditBlocksView"

    @event_handler
    def handle_before_request(cls, e):
        controller = e.source

        if not isinstance(controller.stack_node, EditBlocksNode):
            edit_stack = controller.edit_stack
            edit_stacks_manager = controller.context["edit_stacks_manager"]

            if edit_stack is None:
                edit_stack = edit_stacks_manager.create_edit_stack()
                edit_stacks_manager.current_edit_stack = edit_stack

            node = EditBlocksNode()
            node.item = controller.edited_item
            edit_stack.push(node)
            edit_stacks_manager.preserve_edit_stack(edit_stack)
            edit_stack.go()

    def resolve(self, path):

        try:
            self.edited_item = Item.require_instance(int(path.pop(0)))
        except:
            raise cherrypy.HTTPError(400)

        return self

    @request_property
    def submitted(self):
        return cherrypy.request.method == "POST"

    def submit(self):
        action, selection = self._get_user_action()
        self._invoke_user_action(
            action,
            selection or [self.block or self.block_parent or self.edited_item]
        )

    @request_property
    def block(self):
        return get_parameter(schema.Reference("block", type = Block))

    @request_property
    def block_parent(self):
        return get_parameter(schema.Reference("block_parent", type = Item))

    @request_property
    def block_slot(self):
        return get_slot_parameter(self.block_parent, "block_slot")

    @request_property
    def output(self):
        output = BaseBackOfficeController.output(self)
        output.update(
            edited_item = self.edited_item
        )
        return output


def get_slot_parameter(parent, parameter_name):
    if parent is not None:
        slot_name = get_parameter(schema.String(parameter_name))
        if slot_name:
            slot = type(parent).get_member(slot_name)
            if (
                slot is not None
                and isinstance(slot, schema.RelationMember)
                and slot.related_type
                and issubclass(slot.related_type, Block)
            ):
                return slot




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from woost.controllers.basecmscontroller import BaseCMSController


class PaymentGatewayController(BaseCMSController):

    def __init__(self, payment_gateway):
        self.payment_gateway = payment_gateway
        BaseCMSController.__init__(self)

    def __call__(self, *args, **kwargs):
        raise cherrypy.NotFound()

    @cherrypy.expose
    def handshake(self, **parameters):
        return self.payment_gateway.handle_handshake(parameters)

    @cherrypy.expose
    def notification(self, **parameters):
        payment = self.payment_gateway.process_notification(parameters)
        self.payment_gateway.transaction_notified(payment = payment)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import cherrypy
from cocktail.controllers import (
    Controller,
    FormProcessor,
    Form,
    request_property
)
from woost import app


class SurveyController(FormProcessor, Controller):

    is_transactional = True

    class SurveyForm(Form):

        @request_property
        def survey(self):
            return self.controller.block.survey

        @request_property
        def schema(self):
            return self.survey.create_survey_schema()

        def submit(self):
            Form.submit(self)
            self.survey.add_submission(self.data)

        def after_submit(self):
            if self.controller.block.redirection:
                raise cherrypy.HTTPRedirect(
                    self.controller.block.redirection.get_uri()
                )
            else:
                app.publishable.first_child_redirection()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
from warnings import warn
from woost import app

def get_current_user():
    """Gets the current user for the present context.

    The concept of an 'active' user is used in several places to bind certain
    operations (authorization, versioning...) to an individual.

    @rtype: L{User}
    """
    warn(
        "get_current_user() has been deprecated; use app.user instead",
        DeprecationWarning,
        stacklevel = 2
    )
    return app.user

def set_current_user(user):
    """Sets the current user for the present context. See L{get_current_user}
    for more information.

    @param user: The user to set as the active user.
    @type user: L{User}
    """
    warn(
        "set_current_user() has been deprecated; use app.user instead",
        DeprecationWarning,
        stacklevel = 2
    )
    app.user = user




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Javier Marrero <javier.marrero@whads.com>
"""
from copy import deepcopy
from cocktail.html import templates
RichTextEditor = templates.get_class("woost.views.RichTextEditor")


class NewsletterRichTextEditor(RichTextEditor):

    tinymce_params = deepcopy(RichTextEditor.tinymce_params)

    tinymce_params["plugins"] = " ".join(
        plugin
        for plugin in tinymce_params["plugins"].split()
        if plugin not in ("image", "table")
    )

    tinymce_params["menu"]["insert"]["items"] = " ".join(
        cmd
        for cmd in tinymce_params["menu"]["insert"]["items"].split()
        if cmd != "image"
    )

    tinymce_params["menu"].pop("table", None)




#-*- coding: utf-8 -*-
u"""
(X)HTML templates for the CMS backend application.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from os import path as _path
from pkg_resources import resource_filename
from cocktail.html import Element, resource_repositories
from cocktail.persistence import PersistentClass
from woost.models import Item
from woost.views import uigeneration

def depends_on(self, tag_source, cache_part = None):
    if tag_source is None:
        pass
    elif isinstance(tag_source, basestring):
        tag = tag_source
        if cache_part:
            tag += "-" + cache_part
        self.cache_tags.add(tag)
    elif isinstance(tag_source, Item):
        self.cache_tags.update(
            tag_source.get_cache_tags(cache_part = cache_part)
        )
        self.update_cache_expiration(
            tag_source.get_cache_expiration()
        )
    elif isinstance(tag_source, PersistentClass):
        tag = tag_source.full_name
        if cache_part:
            tag += "-" + cache_part
        self.cache_tags.add(tag)
        self.update_cache_expiration(tag_source.get_cache_expiration_for_type())
    else:
        for item in tag_source:
            self.depends_on(item)

Element.depends_on = depends_on

resource_repositories.define(
    "woost",
    "/resources",
    resource_filename("woost.views", "resources")
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
import re
from urllib2 import urlopen
from json import loads
from cocktail import schema
from cocktail.events import event_handler
from cocktail.controllers import Location
from woost.models import Publishable, Controller

ISSUU_DOCUMENT_URL_PATTERN = \
    re.compile(r"http://issuu.com/(.+)/docs/([^/\?]+)/?(\d+)?(\?e=(\d+/\d+))?.*")

def extract_issuu_document_metadata(url):
    match = re.match(ISSUU_DOCUMENT_URL_PATTERN, url)
    if match:
        return {
            "username": match.group(1),
            "docname": match.group(2),
            "configid": match.group(5)
        }
    else:
        return {}


class IssuuDocument(Publishable):

    instantiable = True
    default_per_language_publication = True
    type_group = "resource"

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(qname = "woost.issuu_document_controller")
    )

    members_order = [
        "title",
        "issuu_document_url",
        "thumbnail_page"
    ]

    title = schema.String(
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True,
        required = True,
        spellcheck = True,
        member_group = "content"
    )

    issuu_document_url = schema.URL(
        required = True,
        format = ISSUU_DOCUMENT_URL_PATTERN,
        searchable = False,
        member_group = "content"
    )

    thumbnail_page = schema.Integer(
        required = True,
        default = 1,
        min = 1,
        member_group = "content",
        listed_by_default = False
    )

    issuu_document_id = schema.String(
        editable = schema.NOT_EDITABLE,
        unique = True,
        indexed = True,
        searchable = False,
        member_group = "content",
        listed_by_default = False
    )

    issuu_config_id = schema.String(
        editable = schema.NOT_EDITABLE,
        searchable = False,
        member_group = "content",
        listed_by_default = False
    )

    def get_issuu_uri(self, page_number = None):
        if page_number == None:
            page_number = self.thumbnail_page

        return u"http://issuu.com/%s/docs/%s/%s?e=%s" % (
            self.issuu_document_username,
            self.issuu_document_name,
            page_number,
            self.issuu_config_id
        )

    def get_uri(self,
        path = None,
        parameters = None,
        language = None,
        host = None,
        encode = True):

        uri = self.get_issuu_uri()

        if uri is not None:

            if path:
                uri = make_uri(uri, *path)

            if parameters:
                uri = make_uri(uri, **parameters)

            uri = self._fix_uri(uri, host, encode)

        return uri

    @event_handler
    def handle_changed(cls, event):

        item = event.source
        member = event.member

        if member.name == "issuu_document_url":
            item._update_issuu_document_metadata()

    def _update_issuu_document_metadata(self, timeout = 5):
        if self.issuu_document_url:
            metadata = extract_issuu_document_metadata(self.issuu_document_url)
            self.issuu_document_username = metadata.get("username")
            self.issuu_document_name = metadata.get("docname")
            self.issuu_config_id = metadata.get("configid")

            if self.issuu_document_username and self.issuu_document_name:
                response = urlopen(
                    "http://search.issuu.com/api/2_0/document?q=docname:%s&username=%s" % (
                        self.issuu_document_name,
                        self.issuu_document_username
                    ),
                    timeout = timeout
                )
                status = response.getcode()
                body = response.read()
                if status < 200 or status > 299:
                    raise IssuuSearchAPIError(body)

                data = loads(body)
                try:
                    self.issuu_document_id = \
                        data["response"]["docs"][0]["documentId"]
                except (KeyError, IndexError):
                    self.issuu_document_id = None

            if self.issuu_config_id is None:
                url = Location(self.issuu_document_url)
                url.query_string.update(e = 0)
                response = urlopen(url.get_url(), timeout = timeout)
                status = response.getcode()
                body = response.read()
                if status >= 200 and status <= 299:
                    url = Location(response.geturl())
                    issuu_config_id = url.query_string.get("e")
                    if issuu_config_id:
                        self.issuu_config_id = issuu_config_id[0]

    def is_internal_content(self, language = None):
        return False


class IssuuSearchAPIError(Exception):

    def __init__(self, response):
        Exception.__init__(self, response)
        self.response = response




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.stringutils import decapitalize
from cocktail.translations import translations
from cocktail.translations.helpers import plural2, ca_possessive, join
from woost.translations.strings \
    import content_permission_translation_factory

translations.define("woost.type_groups.translation_workflow",
    ca = u"Circuït de traducció",
    es = u"Circuito de traducción",
    en = u"Translation workflow"
)

translations.define("woost.actions.translation_workflow_transition",
    ca = u"Canviar estat",
    es = u"Cambiar estado",
    en = u"Change state"
)

translations.define("woost.extensions.translationworkflow.translation_path",
    ca = lambda source_language, target_language, **kwargs:
        ca_possessive(translations("locale", locale = source_language).lower())
        + u" a "
        + translations("locale", locale = target_language).lower(),
    es = lambda source_language, target_language, **kwargs:
        u"De "
        + translations("locale", locale = source_language).lower()
        + u" a "
        + translations("locale", locale = target_language).lower(),
    en = lambda source_language, target_language, **kwargs:
        u"From "
        + translations("locale", locale = source_language).lower()
        + u" to "
        + translations("locale", locale = target_language).lower()
)

translations.define(
    "woost.extensions.translationworkflow.transitionaction."
    "TranslationWorkflowInvalidTextError-instance",
    ca = lambda instance:
        u"<em>%s</em> conté errors que impedeixen passar-la a l'estat "
        u"<em>%s</em> (%s)" % (
            translations(instance.request),
            translations(instance.transition.target_state).lower(),
            translations(instance.error)
        ),
    es = lambda instance:
        u"<em>%s</em> contiene errores que impiden pasarla al estado "
        u"<em>%s</em> (%s)" % (
            translations(instance.request),
            translations(instance.transition.target_state).lower(),
            translations(instance.error)
        ),
    en = lambda instance:
        u"<em>%s</em> contains errors that prevent its transition to the "
        u"<em>%s</em> state (%s)" % (
            translations(instance.request),
            translations(instance.transition.target_state).lower(),
            translations(instance.error)
        )
)

# Configuration
#------------------------------------------------------------------------------
translations.define("Configuration.translation_workflow_paths",
    ca = u"Camins de traducció",
    es = u"Caminos de traducción",
    en = u"Translation paths"
)

translations.define("Configuration.translation_workflow_paths-explanation",
    ca = u"Emparellaments d'idiomes per generar peticions de traducció.",
    es = u"Emparejamientos de idiomas para generar peticiones de traducción.",
    en = u"Language pairings to generate translation requests."
)

# User
#------------------------------------------------------------------------------
translations.define("User.translation_proficiencies",
    ca = u"Traduccions que pot realitzar",
    es = u"Traducciones que puede realizar",
    en = u"Translations he/she can perform"
)

translations.define("User.managed_translation_agency",
    ca = u"Agència de traducció que gestiona",
    es = u"Agencia de traducción que gestiona",
    en = u"Managed translation agency"
)

translations.define("User.translation_agency",
    ca = u"Agència de traducció per la que treballa",
    es = u"Agencia de traducción para la que trabaja",
    en = u"Translation agency he/she works for"
)

# Item
#------------------------------------------------------------------------------
translations.define("Item.translation_requests",
    ca = u"Peticions de traducció",
    es = u"Peticiones de traducción",
    en = u"Translation requests"
)

# Role
#------------------------------------------------------------------------------
translations.define("Role.translation_workflow_relevant_states",
    ca = u"Estats rellevants del circuït de traducció",
    es = u"Estados relevantes del circuito de traducción",
    en = u"Translation workflow relevant states"
)

translations.define("Role.translation_workflow_default_state",
    ca = u"Estat per defecte del circuït de traducció",
    es = u"Estado por defecto del circuito de traducción",
    en = u"Translation workflow default state"
)

translations.define("Role.translation_workflow_default_transition",
    ca = u"Transició per defecte del circuït de traducció",
    es = u"Transición por defecto del circuito de traducción",
    en = u"Translation workflow default transition"
)

translations.define("Role.translation_workflow_default_transition_condition",
    ca = u"Condició per la transició per defecte del circuït de traducció",
    es = u"Condición para la transición por defecto del circuito de traducción",
    en = u"Condition for the translation workflow default transition"
)

# TranslationWorkflowRequest
#------------------------------------------------------------------------------
translations.define("TranslationWorkflowRequest",
    ca = u"Petició de traducció",
    es = u"Petición de traducción",
    en = u"Translation request"
)

translations.define("TranslationWorkflowRequest-plural",
    ca = u"Peticions de traducció",
    es = u"Peticiones de traducción",
    en = u"Translation requests"
)

translations.define("TranslationWorkflowRequest.tabs.all",
    ca = u"Totes",
    es = u"Todas",
    en = u"All"
)

translations.define("TranslationWorkflowRequest.translation_request",
    ca = u"Petició de traducció",
    es = u"Petición de traducción",
    en = u"Translation request"
)

translations.define("TranslationWorkflowRequest.translation_request.info",
    ca = u"Dades de la petició",
    es = u"Datos de la petición",
    en = u"Request data"
)

translations.define("TranslationWorkflowRequest.translation_request.translated_values",
    ca = u"Traducció",
    es = u"Traducción",
    en = u"Translation"
)

translations.define("TranslationWorkflowRequest.translation_request.changelog",
    ca = u"Històric",
    es = u"Histórico",
    en = u"Change log"
)

translations.define("TranslationWorkflowRequest.translated_item",
    ca = u"Element a traduir",
    es = u"Elemento a traducir",
    en = u"Translated item"
)

translations.define("TranslationWorkflowRequest.source_language",
    ca = u"Idioma origen",
    es = u"Idioma origen",
    en = u"Idioma destino"
)

translations.define("TranslationWorkflowRequest.target_language",
    ca = u"Idioma destí",
    es = u"Idioma destino",
    en = u"Target language"
)

translations.define("TranslationWorkflowRequest.state",
    ca = u"Estat",
    es = u"Estado",
    en = u"State"
)

translations.define("TranslationWorkflowRequest.assigned_agency",
    ca = u"Agència de traducció",
    es = u"Agencia de traducción",
    en = u"Translation agency"
)

translations.define("TranslationWorkflowRequest.assigned_translator",
    ca = u"Traductor assignat",
    es = u"Traductor asignado",
    en = u"Assigned translator"
)

translations.define("TranslationWorkflowRequest.translated_values",
    ca = u"Traducció",
    es = u"Traducción",
    en = u"Translation"
)

translations.define("TranslationWorkflowRequest.comments",
    ca = u"Comentaris",
    es = u"Comentarios",
    en = u"Comments"
)

translations.define(
    "woost.extensions.translationworkflow.request."
    "TranslationWorkflowRequest-instance",
    ca = lambda instance, referer = None, **kwargs:
        (
            (
                u"Traducció "
                + ca_possessive(
                     '"%s" '
                     % translations(instance.translated_item)
                )
            )
            if referer is None
            else ""
        )
        + (
            instance.source_language
            and ca_possessive(
                decapitalize(
                    translations("locale", locale = instance.source_language)
                )
            )
            or u"?"
        )
        + u" a "
        + (
            instance.target_language
            and decapitalize(
                translations("locale", locale = instance.target_language)
            )
            or u"?"
        ),
    es = lambda instance, referer = None, **kwargs:
        (
            u'Traducción de "%s" de '
            % translations(instance.translated_item)
            if referer is None
            else u"De "
        )
        + (
            instance.source_language
            and decapitalize(
                translations("locale", locale = instance.source_language)
            )
            or u"?"
        )
        + u" a "
        + (
            decapitalize(
                translations("locale", locale = instance.target_language)
            )
            if instance.target_language
            else "?"
        ),
    en = lambda instance, referer = None, **kwargs:
        (
            u'Translation of "%s" from '
            % translations(instance.translated_item)
            if referer is None
            else u"From "
        )
        + (
            instance.source_language
            and translations("locale", locale = instance.source_language)
            or u"?"
        )
        + u" to "
        + (
            instance.target_language
            and translations("locale", locale = instance.target_language)
            or u"?"
        )
)

translations.define("TranslationWorkflowRequest.msexcel.source",
    ca = u"Text original",
    es = u"Texto original",
    en = u"Source text"
)

translations.define("TranslationWorkflowRequest.msexcel.target",
    ca = u"Traducció",
    es = u"Traducción",
    en = u"Translation"
)

# TranslationWorkflowState
#------------------------------------------------------------------------------
translations.define("TranslationWorkflowState",
    ca = u"Estat del circuït de traducció",
    es = u"Estado del circuito de traducción",
    en = u"Translation workflow state"
)

translations.define("TranslationWorkflowState-plural",
    ca = u"Estats del circuït de traducció",
    es = u"Estados del circuito de traducción",
    en = u"Translation workflow states"
)

translations.define("TranslationWorkflowState.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("TranslationWorkflowState.plural_title",
    ca = u"Nom en plural",
    es = u"Nombre en plural",
    en = u"Plural name"
)

translations.define("TranslationWorkflowState.color",
    ca = u"Color",
    es = u"Color",
    en = u"Color"
)

translations.define("TranslationWorkflowState.incomming_transitions",
    ca = u"Transicions entrants",
    es = u"Transiciones entrantes",
    en = u"Incomming transitions"
)

translations.define("TranslationWorkflowState.outgoing_transitions",
    ca = u"Transicions de sortida",
    es = u"Transiciones de salida",
    en = u"Outgoing transitions"
)

translations.define("TranslationWorkflowState.state_after_source_change",
    ca = u"Nou estat quan es modifiqui el text original",
    es = u"Nuevo estado cuando se modifique el texto original",
    en = u"State after the translated item is modified"
)

translations.define("TranslationWorkflowState.requests",
    ca = u"Peticions de traducció",
    es = u"Peticiones de traducción",
    en = u"Translation requests"
)

# TranslationWorkflowTransition
#------------------------------------------------------------------------------
translations.define("TranslationWorkflowTransition",
    ca = u"Transició del circuït de traducció",
    es = u"Transición del circuito de traducción",
    en = u"Translation workflow transition"
)

translations.define("TranslationWorkflowTransition-plural",
    ca = u"Transicions del circuït de traducció",
    es = u"Transiciones del circuito de traducción",
    en = u"Translation workflow transitions"
)

translations.define("TranslationWorkflowTransition.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("TranslationWorkflowTransition.source_states",
    ca = u"Estat origen",
    es = u"Estado origen",
    en = u"Source state"
)

translations.define("TranslationWorkflowTransition.source_states-explanation",
    ca = u"Els estats on comença la transició. Deixar en blanc per definir "
         u"una transició aplicable a tots els estats.",
    es = u"Los estados desde los que parte la transición. Dejar en blanco "
         u"para definir una transición aplicable a todos los estados.",
    en = u"The starting states for the transition. Leave blank to define a "
         u"transition that can be applied to any state."
)

translations.define("TranslationWorkflowTransition.target_state",
    ca = u"Estat destí",
    es = u"Estado destino",
    en = u"Target state"
)

translations.define("TranslationWorkflowTransition.icon",
    ca = u"Icona",
    es = u"Icono",
    en = u"Icon"
)

translations.define("TranslationWorkflowTransition.relative_order",
    ca = u"Ordre relatiu",
    es = u"Orden relativo",
    en = u"Relative order"
)

translations.define("TranslationWorkflowTransition.requires_valid_text",
    ca = u"Requereix validació del text",
    es = u"Requiere validación del texto",
    en = u"Requires text validation"
)

translations.define("TranslationWorkflowTransition.requires_different_state",
    ca = u"Deshabilitar per peticions a l'estat destí",
    es = u"Deshabilitar para peticiones en el estado destino",
    en = u"Disable for requests in the target state"
)

translations.define("TranslationWorkflowTransition.transition_setup_class",
    ca = u"Codi de preparació de la transició",
    es = u"Código de preparación de la transición",
    en = u"Transition setup code"
)

translations.define("TranslationWorkflowTransition.transition_code",
    ca = u"Codi de transició",
    es = u"Código de transición",
    en = u"Transition code"
)

translations.define(
    "woost.extensions.translationworkflow.transitions_notice",
    ca = lambda new_state, count:
        plural2(
            count,
            u"Una petició passada",
            u"%d peticions passades" % count
        )
        + u" a l'estat <em>%s</em>."
          % decapitalize(translations(new_state)),
    es = lambda new_state, count:
        plural2(
            count,
            u"Una petición pasada",
            u"%d peticiones pasadas" % count
        )
        + u" al estado <em>%s</em>."
          % decapitalize(translations(new_state)),
    en = lambda new_state, count:
        plural2(
            count,
            u"One request",
            u"%d requests" % count
        )
        + u" transitioned to the <em>%s</em> state."
          % decapitalize(translations(new_state))
)

translations.define(
    "woost.extensions.translationworkflow.silence_notice",
    ca = lambda count:
        plural2(
            len(requests),
            u"S'ha ignorat un canvi a una petició silenciada",
            u"S'han ignorat %d canvis a peticions silenciades" % count
        ),
    es = lambda count:
        plural2(
            count,
            u"Se ha ignorado un cambio a una petición silenciada",
            u"Se han ignorado %d cambios a peticiones silenciadas" % count
        ),
    en = lambda count:
        plural2(
            count,
            u"A change to a silenced request has been ignored",
            u"%d changes to silenced requests have been ignored" % count
        )
)

# TranslationAgency
#------------------------------------------------------------------------------
translations.define("TranslationAgency",
    ca = u"Agència de traducció",
    es = u"Agencia de traducción",
    en = u"Translation agency"
)

translations.define("TranslationAgency-plural",
    ca = u"Agències de traducció",
    es = u"Agencias de traducción",
    en = u"Translation agencies"
)

translations.define("TranslationAgency.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("TranslationAgency.managers",
    ca = u"Gestors",
    es = u"Gestores",
    en = u"Managers"
)

translations.define("TranslationAgency.translators",
    ca = u"Traductors",
    es = u"Traductores",
    en = u"Translators"
)

translations.define("TranslationAgency.translation_proficiencies",
    ca = u"Traduccions que pot realitzar",
    es = u"Traducciones que puede realizar",
    en = u"Translations it can perform"
)

# Transition controller
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.translationworkflow.transitionnode."
    "TranslationWorkflowTransitionNode-instance",
    ca = lambda instance:
        instance.transition.title + u" peticions de traducció",
    es = lambda instance:
        instance.transition.title + u" peticiones de traducción",
    en = lambda instance:
        instance.transition.title + u" translation requests"
)

translations.define(
    "woost.extensions.translationworkflow."
    "TranslationWorkflowTransitionView.transition_explanation",
    ca = u"Es canviarà l'estat de les següents peticions:",
    es = u"Se cambiará el estado de las peticiones siguientes:",
    en = u"The state of the following requests will be changed:"
)

translations.define(
    "woost.extensions.translationworkflow."
    "TranslationWorkflowAssignTransitionSetup.assignments",
    ca = u"Assignació de traductors per idiomes",
    es = u"Asignación de traductores por idiomas",
    en = u"Per language translator assignments"
)

# TranslationWorkflowTransitionPermission
#------------------------------------------------------------------------------
translations.define("TranslationWorkflowTransitionPermission",
    ca = u"Permís de transició del circuït de traduccions",
    es = u"Permiso de transición del circuito de traducciones",
    en = u"Translation workflow transition permission"
)

translations.define("TranslationWorkflowTransitionPermission-plural",
    ca = u"Permisos de transició del circuït de traduccions",
    es = u"Permisos de transición del circuito de traducciones",
    en = u"Translation workflow transition permissions"
)

translations.define(
    "woost.extensions.translationworkflow.transitionpermission."
    "TranslationWorkflowTransitionPermission-instance",
    ca = content_permission_translation_factory(
        "ca",
        lambda permission, subject, **kwargs:
            u"aplicar " + (
                join(
                    translations(transition)
                    for transition in permission.transitions
                )
                if permission.transitions else u"canvis d'estat"
            )
            + u" sobre peticions de traducció per " + subject
    ),
    es = content_permission_translation_factory(
        "es",
        lambda permission, subject, **kwargs:
            u"aplicar " + (
                join(
                    translations(transition)
                    for transition in permission.transitions
                )
                if permission.transitions else u"cambios de estado"
            )
            + u" sobre peticiones de traducción para " + subject
    ),
    en = content_permission_translation_factory(
        "en",
        lambda permission, subject, **kwargs:
            u"apply " + (
                join(
                    translations(transition)
                    for transition in permission.transitions
                )
                if permission.transitions else u"state changes"
            )
            + u" to translation requests for " + subject
    )
)

translations.define("TranslationWorkflowTransitionPermission.transitions",
    ca = u"Transicions",
    es = u"Transiciones",
    en = u"Transitions"
)

# TranslationWorkflowComment
#------------------------------------------------------------------------------
translations.define("TranslationWorkflowComment",
    ca = u"Comentari de petició de traducció",
    es = u"Comentario de petición de traducción",
    en = u"Translation request comment"
)

translations.define("TranslationWorkflowComment-plural",
    ca = u"Comentaris de petició de traducció",
    es = u"Comentarios de petición de traducción",
    en = u"Translation request comments"
)

translations.define("TranslationWorkflowComment.request",
    ca = u"Petició",
    es = u"Petición",
    en = u"Request"
)

translations.define("TranslationWorkflowComment.text",
    ca = u"Text",
    es = u"Texto",
    en = u"Text"
)

# TranslationWorkflowTable
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.translationworkflow.TranslationWorkflowTable."
    "missing_translation_notice",
    ca = u"Pendent de traduïr",
    es = u"Pendiente de traducción",
    en = u"Not translated yet"
)

# TranslationWorkflowRequestCard
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.translationworkflow.TranslationWorkflowRequestCard."
    "request_language_field.field_label",
    ca = u"Idioma",
    es = u"Idioma",
    en = u"Language"
)

# TranslationWorkflowPathEditor
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.translationworkflow.TranslationWorkflowPathEditor."
    "source_language_label",
    ca = u"De",
    es = u"De",
    en = u"From"
)

translations.define(
    "woost.extensions.translationworkflow.TranslationWorkflowPathEditor."
    "target_language_label",
    ca = u"a",
    es = u"a",
    en = u"to"
)

# BackOfficeEditViewOverlay
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.translationworkflow.BackOfficeEditViewOverlay."
    "translation_workflow_auto_transition_label",
    ca = lambda transition: u"%s automàticament" % translations(transition),
    es = lambda transition: u"%s automaticamente" % translations(transition),
    en = lambda transition: u"%s automatically" % translations(transition)
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

translations.define("woost.type_groups.custom_forms",
    ca = u"Formularis personalitzats",
    es = u"Formularios personalizados",
    en = u"Custom forms"
)

translations.define("woost.actions.export_form_data",
    ca = u"Exportar les respostes",
    es = u"Exportar las respuestas",
    en = u"Download the answers"
)

translations.define("woost.extensions.forms.submit_date",
    ca = u"Data",
    es = u"Fecha",
    en = u"Date"
)

translations.define("woost.extensions.forms.field_default",
    ca = u"Valor per defecte",
    es = u"Valor por defecto",
    en = u"Default value"
)

translations.define("woost.extensions.forms.field_enumeration",
    ca = u"Valors permesos",
    es = u"Valores permitidos",
    en = u"Allowed values"
)

# FormBlock
#------------------------------------------------------------------------------
translations.define("FormBlock",
    ca = u"Formulari personalitzat",
    es = u"Formulario personalizado",
    en = u"Custom form"
)

translations.define("FormBlock-plural",
    ca = u"Formularis personalitzat",
    es = u"Formularios personalizados",
    en = u"Custom forms"
)

translations.define("FormBlock.form",
    ca = u"Formulari",
    es = u"Formulario",
    en = u"Form"
)

translations.define("FormBlock.field_set",
    ca = u"Conjunt de camps",
    es = u"Conjunto de campos",
    en = u"Field set"
)

translations.define("FormBlock.agreements",
    ca = u"Termes i condicions",
    es = u"Términos y condiciones",
    en = u"Agreements"
)

translations.define("FormBlock.notification_receivers",
    ca = u"Receptors de la notificació",
    es = u"Receptores de la notificación",
    en = u"E-mail notification receivers"
)

translations.define("FormBlock.notification_receivers-explanation",
    ca = u"Indica les adreces de correu electrònic a les que es notificarà "
         u"quan un usuari completi el formulari. Només s'aplica a la "
         u"notificació estàndard: altres notificacions poden tenir altres "
         u"maneres de decidir els seus destinataris.",
    es = u"Indica las direcciones de correo electrónico a las que se "
         u"notificará cuando un usuario complete el formulario. Solo se "
         u"aplica a las notificaciones estándar: otras notificaciones pueden "
         u"tener otras maneras de decidir sus destinatarios.",
    en = u"Indicates the e-mail addresses that should be notified when a user "
         u"submits the form. This only applies to the standard notification: "
         u"other notifications can have different means of deciding their "
         u"receivers."
)

translations.define("FormBlock.email_messages",
    ca = u"Notificacions per correu electrònic",
    es = u"Notificaciones por correo electrónico",
    en = u"E-mail notifications"
)

translations.define("FormBlock.redirection",
    ca = u"Redirecció",
    es = u"Redirección",
    en = u"Redirection"
)

translations.define("FormBlock.redirection-explanation",
    ca = u"Deixar en blanc per redirigir a la primera pàgina filla",
    es = u"Dejar en blanco para redirigr a la primera página hija",
    en = u"Leave blank to redirect to the first child page"
)

translations.define("FormBlock.submit_code",
    ca = u"Codi a executar quan s'envii el formulari",
    es = u"Código a ejecutar cuando se envíe el formulario",
    en = u"Code to execute when the form is submitted"
)

translations.define("FormBlock.after_submit_code",
    ca = u"Codi a executar després que s'hagi enviat el formulari",
    es = u"Código a ejecutar después que se haya enviado el formulario",
    en = u"Code to execute after the form has been submitted"
)

translations.define("FormBlock.view_class",
    ca = u"Vista",
    es = u"Vista",
    en = u"View"
)

# Field
#------------------------------------------------------------------------------
translations.define("Field",
    ca = u"Camp",
    es = u"Campo",
    en = u"Field"
)

translations.define("Field-plural",
    ca = u"Camps",
    es = u"Campos",
    en = u"Fields"
)

translations.define("Field.field",
    ca = u"Camp",
    es = u"Campo",
    en = u"Field"
)

translations.define("Field.field_description",
    ca = u"Descripció",
    es = u"Descripción",
    en = u"Description"
)

translations.define("Field.field_structure",
    ca = u"Estructura",
    es = u"Estructura",
    en = u"Structure"
)

translations.define("Field.field_properties",
    ca = u"Propietats",
    es = u"Propiedades",
    en = u"Properties"
)

translations.define("Field.field_name",
    ca = u"Identificador",
    es = u"Identificador",
    en = u"Identifier"
)

translations.define("Field.field_set",
    ca = u"Conjunt",
    es = u"Conjunto",
    en = u"Field set"
)

translations.define("Field.collection",
    ca = u"Col·lecció",
    es = u"Colección",
    en = u"Collection"
)

translations.define("Field.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("Field.visible_title",
    ca = u"Nom visible",
    es = u"Nombre visible",
    en = u"Visible name"
)

translations.define("Field.empty_label",
    ca = u"Valor indeterminat",
    es = u"Valor indeterminado",
    en = u"Empty label"
)

translations.define("Field.empty_label-explanation",
    ca = u"El text que es mostrarà quan el camp sigui buit",
    es = u"El texto que se mostrará cuando el campo esté vacío",
    en = u"The text for the field's empty option"
)

translations.define("Field.explanation",
    ca = u"Explicació",
    es = u"Explicación",
    en = u"Explanation"
)

translations.define("Field.is_required_field",
    ca = u"Requerit",
    es = u"Requerido",
    en = u"Required"
)

translations.define("Field.field_edit_control",
    ca = u"Interfície d'usuari",
    es = u"Interfaz de usuario",
    en = u"User interface"
)

translations.define(
    "Field.field_edit_control=none",
    ca = u"Control per defecte",
    es = u"Control por defecto",
    en = u"Default control"
)

translations.define("Field.field_edit_control=cocktail.html.TextBox",
    ca = u"Caixa de text",
    es = u"Caja de texto",
    en = u"Text box"
)

translations.define("Field.field_edit_control=cocktail.html.TextArea",
    ca = u"Caixa de text multilínia",
    es = u"Caja de texto multilinea",
    en = u"Text area"
)

translations.define("Field.field_edit_control=cocktail.html.DropdownSelector",
    ca = u"Selector desplegable",
    es = u"Selector desplegable",
    en = u"Dropdown selector"
)

translations.define("Field.field_edit_control=cocktail.html.RadioSelector",
    ca = u"Botons de radio",
    es = u"Botones de radio",
    en = u"Radio buttons"
)

translations.define("Field.field_edit_control=cocktail.html.CheckBox",
    ca = u"Xec",
    es = u"Check",
    en = u"Check box"
)

translations.define("Field.field_edit_control=cocktail.html.CheckList",
    ca = u"Llista de xecs",
    es = u"Lista de checks",
    en = u"Check list"
)

translations.define("Field.field_edit_control=cocktail.html.DatePicker",
    ca = u"Calendari desplegable",
    es = u"Calendario desplegable",
    en = u"Dropdown calendar"
)

translations.define(
    "Field.field_edit_control=cocktail.html.CompoundDateSelector",
    ca = u"Desplegables separats per dia, mes i any",
    es = u"Desplegables separados para día, mes y año",
    en = u"Separate dropdown selectors for day, month and year"
)

translations.define("Field.field_initialization",
    ca = u"Inicialització",
    es = u"Inicialización",
    en = u"Initialization"
)

# FieldSet
#------------------------------------------------------------------------------
translations.define("FieldSet",
    ca = u"Conjunt de camps",
    es = u"Conjunto de campos",
    en = u"Field set"
)

translations.define("FieldSet-plural",
    ca = u"Conjunts de camps",
    es = u"Conjuntos de campos",
    en = u"Field sets"
)

translations.define("FieldSet.base_field_set",
    ca = u"Base",
    es = u"Base",
    en = u"Base"
)

translations.define("FieldSet.derived_field_sets",
    ca = u"Conjunts derivats",
    es = u"Conjuntos derivados",
    en = u"Derived field sets"
)

translations.define("FieldSet.fields",
    ca = u"Camps",
    es = u"Campos",
    en = u"Fields"
)

# CollectionField
#------------------------------------------------------------------------------
translations.define("CollectionField",
    ca = u"Camp de col·lecció",
    es = u"Campo de colección",
    en = u"Collection field"
)

translations.define("CollectionField-plural",
    ca = u"Camps de col·lecció",
    es = u"Campos de colección",
    en = u"Collection fields"
)

translations.define("CollectionField.items",
    ca = u"Elements de la col·lecció",
    es = u"Elementos de la colección",
    en = u"Collection items"
)

translations.define("CollectionField.min",
    ca = u"Nombre mínim d'elements",
    es = u"Número mínimo de elementos",
    en = u"Minimum number of items"
)

translations.define("CollectionField.max",
    ca = u"Nombre màxim d'elements",
    es = u"Número máximo de elementos",
    en = u"Maximum number of items"
)

# BooleanField
#------------------------------------------------------------------------------
translations.define("BooleanField",
    ca = u"Camp de sí / no",
    es = u"Campo de sí / no",
    en = u"Yes / no field"
)

translations.define("BooleanField-plural",
    ca = u"Camps de sí / no",
    es = u"Campos de sí / no",
    en = u"Yes / no fields"
)

# TextField
#------------------------------------------------------------------------------
translations.define("TextField",
    ca = u"Camp de text",
    es = u"Campo de texto",
    en = u"Text field"
)

translations.define("TextField-plural",
    ca = u"Camps de text",
    es = u"Campos de texto",
    en = u"Text fields"
)

translations.define("TextField.format",
    ca = u"Format",
    es = u"Formato",
    en = u"Format"
)

translations.define("TextField.min",
    ca = u"Longitud mínima",
    es = u"Longitud mínima",
    en = u"Minimum length"
)

translations.define("TextField.max",
    ca = u"Longitud màxima",
    es = u"Longitud máxima",
    en = u"Maximum length"
)

# OptionsField
#------------------------------------------------------------------------------
translations.define("OptionsField",
    ca = u"Camp d'opcions",
    es = u"Campo de opciones",
    en = u"Options field"
)

translations.define("OptionsField-plural",
    ca = u"Camps d'opcions",
    es = u"Campos de opciones",
    en = u"Option fields"
)

translations.define("OptionsField.options",
    ca = u"Opcions",
    es = u"Opciones",
    en = u"Options"
)

# OptionsFieldOption
#------------------------------------------------------------------------------
translations.define("OptionsFieldOption",
    ca = u"Entrada de camp d'opcions",
    es = u"Entrada de campo de opciones",
    en = u"Options field entry"
)

translations.define("OptionsFieldOption-plural",
    ca = u"Entrades de camp d'opcions",
    es = u"Entradas de campo de opciones",
    en = u"Options field entries"
)

translations.define("OptionsFieldOption.field",
    ca = u"Camp",
    es = u"Campo",
    en = u"Name"
)

translations.define("OptionsFieldOption.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

# EmailAddressField
#------------------------------------------------------------------------------
translations.define("EmailAddressField",
    ca = u"Camp d'adreça de correu electrònic",
    es = u"Campo de dirección de correo electrónico",
    en = u"E-mail address field"
)

translations.define("EmailAddressField-plural",
    ca = u"Camps d'adreça de correu electrònic",
    es = u"Campos de dirección de correo electrónico",
    en = u"E-mail address fields"
)

# PhoneNumberField
#------------------------------------------------------------------------------
translations.define("PhoneNumberField",
    ca = u"Camp de número de telèfon",
    es = u"Campo de número de teléfono",
    en = u"Phone number field"
)

translations.define("PhoneNumberField-plural",
    ca = u"Camps de número de telèfon",
    es = u"Campos de número de teléfono",
    en = u"Phone number fields"
)

# HTMLField
#------------------------------------------------------------------------------
translations.define("HTMLField",
    ca = u"Camp de text enriquit",
    es = u"Campo de texto enriquecido",
    en = u"Rich text field"
)

translations.define("HTMLField-plural",
    ca = u"Camps de text enriquit",
    es = u"Campos de texto enriquecido",
    en = u"Rich text fields"
)

# NumericField
#------------------------------------------------------------------------------
translations.define("NumericField",
    ca = u"Camp numèric",
    es = u"Campo numérico",
    en = u"Numeric field"
)

translations.define("NumericField-plural",
    ca = u"Camps numèrics",
    es = u"Campos numéricos",
    en = u"Numeric fields"
)

translations.define("NumericField.min",
    ca = u"Mínim",
    es = u"Mínimo",
    en = u"Minimum"
)

translations.define("NumericField.max",
    ca = u"Màxim",
    es = u"Máximo",
    en = u"Maximum"
)

# IntegerField
#------------------------------------------------------------------------------
translations.define("IntegerField",
    ca = u"Camp enter",
    es = u"Campo entero",
    en = u"Integer field"
)

translations.define("IntegerField-plural",
    ca = u"Camps enters",
    es = u"Campos enteros",
    en = u"Integer fields"
)

# DecimalField
#------------------------------------------------------------------------------
translations.define("DecimalField",
    ca = u"Camp decimal",
    es = u"Campo decimal",
    en = u"Decimal field"
)

translations.define("DecimalField-plural",
    ca = u"Camps decimals",
    es = u"Campos decimales",
    en = u"Decimal fields"
)

# DateField
#------------------------------------------------------------------------------
translations.define("DateField",
    ca = u"Camp de data",
    es = u"Campo de fecha",
    en = u"Date field"
)

translations.define("DateField-plural",
    ca = u"Camps de data",
    es = u"Campos de fecha",
    en = u"Date fields"
)

# TimeField
#------------------------------------------------------------------------------
translations.define("TimeField",
    ca = u"Camp d'hora",
    es = u"Campo de hora",
    en = u"Time field"
)

translations.define("TimeField-plural",
    ca = u"Camps d'hora",
    es = u"Campos de hora",
    en = u"Time fields"
)

# DateTimeField
#------------------------------------------------------------------------------
translations.define("DateTimeField",
    ca = u"Camp de data / hora",
    es = u"Campo de fecha / hora",
    en = u"Date / time field"
)

translations.define("DateTimeField-plural",
    ca = u"Camps de data / hora",
    es = u"Campos de fecha / hora",
    en = u"Date / time fields"
)

# FormAgreement
#------------------------------------------------------------------------------
translations.define("FormAgreement",
    ca = u"Termes i condicions",
    es = u"Términos y condiciones",
    en = u"Terms and conditions"
)

translations.define("FormAgreement-plural",
    ca = u"Termes i condicions",
    es = u"Términos y condiciones",
    en = u"Terms and conditions"
)

translations.define("FormAgreement.forms",
    ca = u"Formularis",
    es = u"Formularios",
    en = u"Forms"
)

translations.define("FormAgreement.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("FormAgreement.document",
    ca = u"Document",
    es = u"Documento",
    en = u"Document"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from woost.models import Item, User
from .path import TranslationWorkflowPath


class TranslationAgency(Item):

    type_group = "users"

    members_order = [
        "title",
        "managers",
        "translators",
        "translation_proficiencies"
    ]

    title = schema.String(
        required = True,
        indexed = True,
        unique = True,
        descriptive = True
    )

    managers = schema.Collection(
        items = schema.Reference(type = User),
        bidirectional = True,
        related_key = "managed_translation_agency"
    )

    translators = schema.Collection(
        items = schema.Reference(type = User),
        bidirectional = True,
        related_key = "translation_agency"
    )

    translation_proficiencies = schema.Collection(
        items = TranslationWorkflowPath
    )

